from __future__ import annotations

from enum import Enum

__all__ = [
    "ContractVersion",
]


class ContractVersion(str, Enum):
    """Standard TON smart contract version identifier."""

    WalletV1R1 = "wallet_v1r1"
    """Wallet contract version 1, revision 1."""
    WalletV1R2 = "wallet_v1r2"
    """Wallet contract version 1, revision 2."""
    WalletV1R3 = "wallet_v1r3"
    """Wallet contract version 1, revision 3."""
    WalletV2R1 = "wallet_v2r1"
    """Wallet contract version 2, revision 1."""
    WalletV2R2 = "wallet_v2r2"
    """Wallet contract version 2, revision 2."""
    WalletV3R1 = "wallet_v3r1"
    """Wallet contract version 3, revision 1."""
    WalletV3R2 = "wallet_v3r2"
    """Wallet contract version 3, revision 2."""
    WalletV4R1 = "wallet_v4r1"
    """Wallet contract version 4, revision 1."""
    WalletV4R2 = "wallet_v4r2"
    """Wallet contract version 4, revision 2."""
    WalletV5Beta = "wallet_v5_beta"
    """Wallet contract version 5, beta release."""
    WalletV5R1 = "wallet_v5r1"
    """Wallet contract version 5, revision 1."""

    WalletHighloadV2 = "wallet_highload_v2"
    """Highload wallet contract version 2."""
    WalletHighloadV3R1 = "wallet_highload_v3r1"
    """Highload wallet contract version 3, revision 1."""

    WalletPreprocessedV2 = "wallet_preprocessed_v2"
    """Preprocessed wallet contract version 2."""

    NFTCollectionStandard = "nft_collection_standard"
    """Standard NFT collection contract."""
    NFTCollectionEditable = "nft_collection_editable"
    """Editable NFT collection contract."""
    NFTItemStandard = "nft_item_standard"
    """Standard NFT item contract."""
    NFTItemEditable = "nft_item_editable"
    """Editable NFT item contract."""
    NFTItemSoulbound = "nft_item_soulbound"
    """Soulbound NFT item contract."""

    JettonMasterStandard = "jetton_master_standard"
    """Standard jetton master contract."""
    JettonMasterStablecoin = "jetton_master_stablecoin"
    """Stablecoin jetton master contract."""
    JettonMasterStablecoinV2 = "jetton_master_stablecoin_v2"
    """Stablecoin jetton master contract, version 2."""
    JettonWalletStandard = "jetton_wallet_standard"
    """Standard jetton wallet contract."""
    JettonWalletStablecoin = "jetton_wallet_stablecoin"
    """Stablecoin jetton wallet contract."""
    JettonWalletStablecoinV2 = "jetton_wallet_stablecoin_v2"
    """Stablecoin jetton wallet contract, version 2."""

    TelegramUsernamesCollection = "telegram_usernames_collection"
    """Telegram usernames NFT collection contract."""
    TelegramGiftsCollection = "telegram_gifts_collection"
    """Telegram gifts NFT collection contract."""
    TelegramUsernameItem = "telegram_username_item"
    """Telegram username NFT item contract."""
    TelegramGiftItem = "telegram_gift_item"
    """Telegram gift NFT item contract."""

    TONDNSCollection = "ton_dns_collection"
    """TON DNS collection contract."""
    TONDNSItem = "ton_dns_item"
    """TON DNS item contract."""




from pytoniq_core.boc.tvm_bitarray import (
    BitarrayLike,
    BytesLike,
    TvmBitarray,
    TvmBitarrayException,
    TvmBitarrayOverflowException,
    TvmBitarrayUnderflowException,
)

__all__ = [
    "BitarrayLike",
    "BytesLike",
    "TvmBitarray",
    "TvmBitarrayException",
    "TvmBitarrayOverflowException",
    "TvmBitarrayUnderflowException",
]



from collections.abc import Callable
from typing import Any, TypeVar

from ...boc.address import Address
from ...boc.cell import Cell
from ...boc.slice import Slice

Key = int | str | bytes | Address

_H = TypeVar("_H", bound="HashMap")


class DictError(Exception): ...


class HashMap:
    size: int
    map: dict[int, Any]
    key_serializer: Callable[..., Any] | None
    value_serializer: Callable[..., Any] | None

    def __init__(
        self,
        key_size: int,
        key_serializer: Callable[..., Any] | None = ...,
        value_serializer: Callable[..., Any] | None = ...,
        map_: dict[int, Any] | None = ...,
    ) -> None: ...
    def set_int_key(self: _H, int_key: int, value: Any) -> _H: ...
    def set(self: _H, key: Key, value: Any, hash_key: bool = ...) -> _H: ...
    def with_address_values(self: _H) -> _H: ...
    def with_uint_values(self: _H, length: int) -> _H: ...
    def with_int_values(self: _H, length: int) -> _H: ...
    def with_coins_values(self: _H) -> _H: ...
    def serialize(self) -> Cell | None: ...
    @classmethod
    def from_cell(cls, dict_cell: Cell, key_length: int) -> HashMap: ...
    @staticmethod
    def parse(
        dict_cell: Slice,
        key_length: int,
        key_deserializer: Callable[..., Any] | None = ...,
        value_deserializer: Callable[..., Any] | None = ...,
    ) -> dict[Any, Any] | None: ...



from __future__ import annotations

from typing import Any, TypeVar

from .builder import Builder as Builder
from .deserialize import NullCell
from .exotic import LevelMask
from .slice import Slice as Slice
from .tvm_bitarray import BitarrayLike, TvmBitarray

_CellT = TypeVar("_CellT", bound="Cell")

class CellError(Exception): ...

class Cell(NullCell):
    bits: TvmBitarray
    refs: list[Cell]
    type_: int
    is_exotic: bool
    level_mask: LevelMask

    def __init__(
        self,
        bits: BitarrayLike,
        refs: list[Cell],
        cell_type: int = ...,
    ) -> None: ...
    @classmethod
    def empty(cls: type[_CellT]) -> _CellT: ...
    def resolve_mask(self) -> LevelMask: ...
    def to_builder(self) -> Builder: ...
    def to_slice(self) -> Slice: ...
    def get_refs_descriptor(self, lvl_mask: LevelMask) -> bytes: ...
    def get_bits_descriptor(self) -> bytes: ...
    def get_descriptors(self, lvl_mask: LevelMask = ...) -> bytes: ...
    def get_depth(self, lvl_mask: int = ...) -> int: ...
    def get_data_bytes(self) -> bytes: ...
    def get_representation(self) -> bytes: ...
    @property
    def hash(self) -> bytes: ...
    @property
    def data(self) -> bytes: ...
    def get_hash(self, lvl_mask: int) -> bytes: ...
    def calculate_hashes(self) -> None: ...
    def calculate_representation_hash(self) -> bytes: ...
    def order(self, result: dict[Any, Any] = ...) -> dict[Any, Any]: ...
    def serialize(self, indexes: dict[Any, Any], byte_len: int) -> bytes: ...
    def to_boc(
        self,
        has_idx: bool = ...,
        hash_crc32: bool = ...,
        has_cache_bits: bool = ...,
        flags: int = ...,
    ) -> bytes: ...
    @classmethod
    def from_boc(cls: type[_CellT], data: Any) -> list[_CellT]: ...
    @classmethod
    def one_from_boc(cls: type[_CellT], data: Any) -> _CellT: ...
    def begin_parse(self) -> Slice: ...
    def copy(self) -> Cell: ...
    def __hash__(self) -> int: ...
    def __getitem__(self, ref_i: int) -> Cell: ...
    def __eq__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...



from pytoniq_core.boc.hashmap.parse import (
    deserialize_hashmap_aug_node,
    deserialize_hashmap_node,
    deserialize_hml,
    deserialize_unary,
    parse,
    parse_aug,
    parse_hashmap,
    parse_hashmap_aug,
    read_arbitrary_uint,
)

__all__ = [
    "deserialize_hashmap_aug_node",
    "deserialize_hashmap_node",
    "deserialize_hml",
    "deserialize_unary",
    "parse",
    "parse_aug",
    "parse_hashmap",
    "parse_hashmap_aug",
    "read_arbitrary_uint",
]



from pytoniq_core.tlb.config import (
    BlockCreateFees,
    BlockLimits,
    CatchainConfig,
    ComplaintPricing,
    ConfigError,
    ConfigParam,
    ConfigParam0,
    ConfigParam1,
    ConfigParam2,
    ConfigParam3,
    ConfigParam4,
    ConfigParam5,
    ConfigParam6,
    ConfigParam7,
    ConfigParam8,
    ConfigParam9,
    ConfigParam10,
    ConfigParam11,
    ConfigParam12,
    ConfigParam13,
    ConfigParam14,
    ConfigParam15,
    ConfigParam16,
    ConfigParam17,
    ConfigParam18,
    ConfigParam20,
    ConfigParam21,
    ConfigParam22,
    ConfigParam23,
    ConfigParam24,
    ConfigParam25,
    ConfigParam28,
    ConfigParam29,
    ConfigParam31,
    ConfigParam32,
    ConfigParam33,
    ConfigParam34,
    ConfigParam35,
    ConfigParam36,
    ConfigParam37,
    ConfigParam44,
    ConfigParam71,
    ConfigParam72,
    ConfigParam73,
    ConfigParam79,
    ConfigParam81,
    ConfigParam82,
    ConfigProposalSetup,
    ConfigVotingSetup,
    ConsensusConfig,
    GasLimitsPrices,
    JettonBridgeParams,
    JettonBridgePrices,
    MsgForwardPrices,
    OracleBridgeParams,
    ParamLimits,
    SigPubKey,
    StoragePrices,
    SuspendedAddressList,
    ValidatorDescr,
    ValidatorSet,
    WcSplitMergeTimings,
    WorkchainDescr,
    WorkchainFormat,
)

__all__ = [
    "BlockCreateFees",
    "BlockLimits",
    "CatchainConfig",
    "ComplaintPricing",
    "ConfigError",
    "ConfigParam",
    "ConfigParam0",
    "ConfigParam1",
    "ConfigParam2",
    "ConfigParam3",
    "ConfigParam4",
    "ConfigParam5",
    "ConfigParam6",
    "ConfigParam7",
    "ConfigParam8",
    "ConfigParam9",
    "ConfigParam10",
    "ConfigParam11",
    "ConfigParam12",
    "ConfigParam13",
    "ConfigParam14",
    "ConfigParam15",
    "ConfigParam16",
    "ConfigParam17",
    "ConfigParam18",
    "ConfigParam20",
    "ConfigParam21",
    "ConfigParam22",
    "ConfigParam23",
    "ConfigParam24",
    "ConfigParam25",
    "ConfigParam28",
    "ConfigParam29",
    "ConfigParam31",
    "ConfigParam32",
    "ConfigParam33",
    "ConfigParam34",
    "ConfigParam35",
    "ConfigParam36",
    "ConfigParam37",
    "ConfigParam44",
    "ConfigParam71",
    "ConfigParam72",
    "ConfigParam73",
    "ConfigParam79",
    "ConfigParam81",
    "ConfigParam82",
    "ConfigProposalSetup",
    "ConfigVotingSetup",
    "ConsensusConfig",
    "GasLimitsPrices",
    "JettonBridgeParams",
    "JettonBridgePrices",
    "MsgForwardPrices",
    "OracleBridgeParams",
    "ParamLimits",
    "SigPubKey",
    "StoragePrices",
    "SuspendedAddressList",
    "ValidatorDescr",
    "ValidatorSet",
    "WcSplitMergeTimings",
    "WorkchainDescr",
    "WorkchainFormat",
]



from ton_core.boc.address import Address
from ton_core.boc.cell import Cell
from ton_core.tl.block import BlockIdExt
from ton_core.tlb.account import ShardAccount
from ton_core.tlb.block import ShardDescr
from ton_core.tlb.config import CatchainConfig, ValidatorDescr, ValidatorSet

class ProofError(Exception): ...

def calculate_node_id_short(pub_key: bytes) -> bytes: ...
def check_proof(cell: Cell, hash_: bytes) -> None: ...
def check_block_header_proof(
    root_cell: Cell,
    block_hash: bytes,
    store_state_hash: bool = ...,
) -> bytes | None: ...
def check_shard_proof(
    shard_proof: bytes,
    blk: BlockIdExt,
    shrd_blk: BlockIdExt,
) -> ShardDescr | None: ...
def check_account_proof(
    proof: bytes,
    shrd_blk: BlockIdExt,
    address: Address,
    account_state_root: Cell,
    return_account_descr: bool = ...,
) -> ShardAccount | None: ...
def check_block_signatures(
    nodes: list[ValidatorDescr],
    signatures: list[dict[str, object]],
    blk: BlockIdExt,
) -> None: ...
def compute_validator_set(
    ccv_conf: CatchainConfig,
    blk: BlockIdExt,
    vset: ValidatorSet,
    cc_seqno: int | None = ...,
) -> list[ValidatorDescr]: ...



from ton_core.contrib.contracts.nft.tlb import (
    NFTCollectionBatchMintItemBody,
    NFTCollectionChangeContentBody,
    NFTCollectionChangeOwnerBody,
    NFTCollectionContent,
    NFTCollectionData,
    NFTCollectionMintItemBody,
    NFTDestroyBody,
    NFTEditContentBody,
    NFTItemEditableData,
    NFTItemEditableMintRef,
    NFTItemSoulboundData,
    NFTItemSoulboundMintRef,
    NFTItemStandardData,
    NFTItemStandardMintRef,
    NFTRevokeBody,
    NFTTransferBody,
    NFTTransferEditorshipBody,
    OffchainCommonContent,
    OffchainContent,
    OffchainItemContent,
    OnchainContent,
    RoyaltyParams,
)

__all__ = [
    "NFTCollectionBatchMintItemBody",
    "NFTCollectionChangeContentBody",
    "NFTCollectionChangeOwnerBody",
    "NFTCollectionContent",
    "NFTCollectionData",
    "NFTCollectionMintItemBody",
    "NFTDestroyBody",
    "NFTEditContentBody",
    "NFTItemEditableData",
    "NFTItemEditableMintRef",
    "NFTItemSoulboundData",
    "NFTItemSoulboundMintRef",
    "NFTItemStandardData",
    "NFTItemStandardMintRef",
    "NFTRevokeBody",
    "NFTTransferBody",
    "NFTTransferEditorshipBody",
    "OffchainCommonContent",
    "OffchainContent",
    "OffchainItemContent",
    "OnchainContent",
    "RoyaltyParams",
]



from .codes import CONTRACT_CODES
from .dns import (
    BaseDNSRecord,
    BaseDNSRecordAddress,
    BaseDNSRecordBinary,
    ChangeDNSRecordBody,
    DNSBalanceReleaseBody,
    DNSRecordDNSNextResolver,
    DNSRecords,
    DNSRecordSite,
    DNSRecordStorage,
    DNSRecordText,
    DNSRecordWallet,
    RenewDNSBody,
    TONDNSAuction,
    TONDNSCollectionData,
    TONDNSItemData,
)
from .jetton import (
    JettonBurnBody,
    JettonChangeAdminBody,
    JettonChangeContentBody,
    JettonClaimAdminBody,
    JettonDiscoveryBody,
    JettonDropAdminBody,
    JettonInternalTransferBody,
    JettonMasterStablecoinData,
    JettonMasterStandardData,
    JettonMintBody,
    JettonStandardChangeAdminBody,
    JettonStandardChangeContentBody,
    JettonStandardMintBody,
    JettonTopUpBody,
    JettonTransferBody,
    JettonUpgradeBody,
    JettonWalletStablecoinData,
    JettonWalletStablecoinV2Data,
    JettonWalletStandardData,
)
from .nft import (
    NFTCollectionBatchMintItemBody,
    NFTCollectionChangeContentBody,
    NFTCollectionChangeOwnerBody,
    NFTCollectionContent,
    NFTCollectionData,
    NFTCollectionMintItemBody,
    NFTDestroyBody,
    NFTEditContentBody,
    NFTItemEditableData,
    NFTItemEditableMintRef,
    NFTItemSoulboundData,
    NFTItemSoulboundMintRef,
    NFTItemStandardData,
    NFTItemStandardMintRef,
    NFTRevokeBody,
    NFTTransferBody,
    NFTTransferEditorshipBody,
    OffchainCommonContent,
    OffchainContent,
    OffchainItemContent,
    OnchainContent,
    RoyaltyParams,
)
from .opcodes import OpCode
from .telegram import (
    TeleCollectionData,
    TeleItemAuction,
    TeleItemAuctionConfig,
    TeleItemAuctionState,
    TeleItemCancelAuctionBody,
    TeleItemConfig,
    TeleItemContent,
    TeleItemData,
    TeleItemStartAuctionBody,
    TeleItemState,
    TeleItemTokenInfo,
)
from .vanity import (
    VanityDeployBody,
)
from .versions import ContractVersion
from .wallet import (
    BaseContractConfig,
    BaseWalletConfig,
    BaseWalletData,
    BaseWalletParams,
    EncryptedTextCommentBody,
    OutActionSendMsg,
    TextCommentBody,
    WalletHighloadV2Config,
    WalletHighloadV2Data,
    WalletHighloadV2Params,
    WalletHighloadV3Config,
    WalletHighloadV3Data,
    WalletHighloadV3Params,
    WalletPreprocessedV2Config,
    WalletPreprocessedV2Data,
    WalletPreprocessedV2Params,
    WalletV1Config,
    WalletV1Data,
    WalletV1Params,
    WalletV2Config,
    WalletV2Data,
    WalletV2Params,
    WalletV3Config,
    WalletV3Data,
    WalletV3Params,
    WalletV4Config,
    WalletV4Data,
    WalletV4Params,
    WalletV5BetaConfig,
    WalletV5BetaData,
    WalletV5BetaParams,
    WalletV5Config,
    WalletV5Data,
    WalletV5Params,
    WalletV5SubwalletID,
)

__all__ = [
    "CONTRACT_CODES",
    "BaseContractConfig",
    "BaseDNSRecord",
    "BaseDNSRecordAddress",
    "BaseDNSRecordBinary",
    "BaseWalletConfig",
    "BaseWalletData",
    "BaseWalletParams",
    "ChangeDNSRecordBody",
    "ContractVersion",
    "DNSBalanceReleaseBody",
    "DNSRecordDNSNextResolver",
    "DNSRecordSite",
    "DNSRecordStorage",
    "DNSRecordText",
    "DNSRecordWallet",
    "DNSRecords",
    "EncryptedTextCommentBody",
    "JettonBurnBody",
    "JettonChangeAdminBody",
    "JettonChangeContentBody",
    "JettonClaimAdminBody",
    "JettonDiscoveryBody",
    "JettonDropAdminBody",
    "JettonInternalTransferBody",
    "JettonMasterStablecoinData",
    "JettonMasterStandardData",
    "JettonMintBody",
    "JettonStandardChangeAdminBody",
    "JettonStandardChangeContentBody",
    "JettonStandardMintBody",
    "JettonTopUpBody",
    "JettonTransferBody",
    "JettonUpgradeBody",
    "JettonWalletStablecoinData",
    "JettonWalletStablecoinV2Data",
    "JettonWalletStandardData",
    "NFTCollectionBatchMintItemBody",
    "NFTCollectionChangeContentBody",
    "NFTCollectionChangeOwnerBody",
    "NFTCollectionContent",
    "NFTCollectionData",
    "NFTCollectionMintItemBody",
    "NFTDestroyBody",
    "NFTEditContentBody",
    "NFTItemEditableData",
    "NFTItemEditableMintRef",
    "NFTItemSoulboundData",
    "NFTItemSoulboundMintRef",
    "NFTItemStandardData",
    "NFTItemStandardMintRef",
    "NFTRevokeBody",
    "NFTTransferBody",
    "NFTTransferEditorshipBody",
    "OffchainCommonContent",
    "OffchainContent",
    "OffchainItemContent",
    "OnchainContent",
    "OpCode",
    "OutActionSendMsg",
    "RenewDNSBody",
    "RoyaltyParams",
    "TONDNSAuction",
    "TONDNSCollectionData",
    "TONDNSItemData",
    "TeleCollectionData",
    "TeleItemAuction",
    "TeleItemAuctionConfig",
    "TeleItemAuctionState",
    "TeleItemCancelAuctionBody",
    "TeleItemConfig",
    "TeleItemContent",
    "TeleItemData",
    "TeleItemStartAuctionBody",
    "TeleItemState",
    "TeleItemTokenInfo",
    "TextCommentBody",
    "VanityDeployBody",
    "WalletHighloadV2Config",
    "WalletHighloadV2Data",
    "WalletHighloadV2Params",
    "WalletHighloadV3Config",
    "WalletHighloadV3Data",
    "WalletHighloadV3Params",
    "WalletPreprocessedV2Config",
    "WalletPreprocessedV2Data",
    "WalletPreprocessedV2Params",
    "WalletV1Config",
    "WalletV1Data",
    "WalletV1Params",
    "WalletV2Config",
    "WalletV2Data",
    "WalletV2Params",
    "WalletV3Config",
    "WalletV3Data",
    "WalletV3Params",
    "WalletV4Config",
    "WalletV4Data",
    "WalletV4Params",
    "WalletV5BetaConfig",
    "WalletV5BetaData",
    "WalletV5BetaParams",
    "WalletV5Config",
    "WalletV5Data",
    "WalletV5Params",
    "WalletV5SubwalletID",
]



from dataclasses import dataclass

from .cell import Cell

@dataclass
class Anycast:
    depth: int
    rewrite_pfx: int

class AddressError(Exception): ...

class Address:
    wc: int
    hash_part: bytes
    is_bounceable: bool
    is_test_only: bool
    anycast: Anycast | None

    def __init__(self, address: tuple[int, bytes] | Address | str) -> None: ...
    def set_anycast(self, depth: int, rewrite_pfx: int) -> None: ...
    def is_hex(self, addr: str) -> bool: ...
    def is_b64(self, addr: str) -> bool: ...
    def to_str(
        self,
        is_user_friendly: bool = ...,
        is_url_safe: bool = ...,
        is_bounceable: bool = ...,
        is_test_only: bool = ...,
    ) -> str: ...
    def to_tl_account_id(self) -> dict[str, int | str]: ...
    def to_cell(self) -> Cell: ...
    def __eq__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...

class ExternalAddress:
    external_address: int | None
    len: int

    def __init__(
        self,
        address: int | str | bytes | None,
        length: int | None = ...,
    ) -> None: ...
    def to_cell(self) -> Cell: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...



from .address import (
    Address,
    AddressError,
    Anycast,
    ExternalAddress,
)
from .builder import (
    Builder,
)
from .cell import (
    Cell,
    CellError,
)
from .exotic import (
    CellTypes,
    LevelMask,
)
from .hashmap import (
    DictError,
    HashMap,
    Key,
)
from .slice import (
    Slice,
    SliceError,
)
from .tvm_bitarray import (
    BitarrayLike,
    BytesLike,
    TvmBitarray,
    TvmBitarrayException,
    TvmBitarrayOverflowException,
    TvmBitarrayUnderflowException,
)


def begin_cell() -> Builder:
    return Builder()


__all__ = [
    "Address",
    "AddressError",
    "Anycast",
    "BitarrayLike",
    "Builder",
    "BytesLike",
    "Cell",
    "CellError",
    "CellTypes",
    "DictError",
    "ExternalAddress",
    "HashMap",
    "Key",
    "LevelMask",
    "Slice",
    "SliceError",
    "TvmBitarray",
    "TvmBitarrayException",
    "TvmBitarrayOverflowException",
    "TvmBitarrayUnderflowException",
    "begin_cell",
]



from pytoniq_core.tlb.transaction import (
    AccStatusChange,
    CommonMsgInfo,
    ComputeSkipReason,
    ExternalMsgInfo,
    ExternalOutMsgInfo,
    ImportFees,
    InMsg,
    IntermediateAddress,
    InternalMsgInfo,
    LibRef,
    MessageAny,
    MsgEnvelope,
    MsgMetadata,
    OutAction,
    OutList,
    OutMsg,
    SplitMergeInfo,
    TrActionPhase,
    Transaction,
    TransactionDescr,
    TransactionError,
    TransactionMergeInstall,
    TransactionMergePrepare,
    TransactionOrdinary,
    TransactionSplitInstall,
    TransactionSplitPrepare,
    TransactionStorage,
    TransactionTickTock,
    TrBouncePhase,
    TrComputePhase,
    TrCreditPhase,
    TrStoragePhase,
)

__all__ = [
    "AccStatusChange",
    "CommonMsgInfo",
    "ComputeSkipReason",
    "ExternalMsgInfo",
    "ExternalOutMsgInfo",
    "ImportFees",
    "InMsg",
    "IntermediateAddress",
    "InternalMsgInfo",
    "LibRef",
    "MessageAny",
    "MsgEnvelope",
    "MsgMetadata",
    "OutAction",
    "OutList",
    "OutMsg",
    "SplitMergeInfo",
    "TrActionPhase",
    "TrBouncePhase",
    "TrComputePhase",
    "TrCreditPhase",
    "TrStoragePhase",
    "Transaction",
    "TransactionDescr",
    "TransactionError",
    "TransactionMergeInstall",
    "TransactionMergePrepare",
    "TransactionOrdinary",
    "TransactionSplitInstall",
    "TransactionSplitPrepare",
    "TransactionStorage",
    "TransactionTickTock",
]



from collections.abc import Callable
from typing import Any

from bitarray import bitarray

from ...boc.slice import Slice

def read_arbitrary_uint(n: int, ser: bitarray) -> tuple[int, bitarray]: ...
def deserialize_unary(ser: Slice) -> int: ...
def deserialize_hml(ser: Slice, m: int) -> tuple[int, bitarray]: ...
def deserialize_hashmap_node(cs: Slice, m: int, ret_dict: dict[str, Any], prefix: bitarray) -> None: ...
def deserialize_hashmap_aug_node(
    cs: Slice,
    m: int,
    ret_dict: dict[str, Any],
    extras: list[Any],
    prefix: bitarray,
    x_deserializer: Callable[..., Any],
    y_deserializer: Callable[..., Any],
) -> None: ...
def parse(slice: Slice, key_length: int, ret_dict: dict[str, Any], prefix: bitarray) -> None: ...
def parse_aug(
    slice: Slice,
    key_length: int,
    ret_dict: dict[str, Any],
    extras: list[Any],
    prefix: bitarray,
    x_deserializer: Callable[..., Any],
    y_deserializer: Callable[..., Any],
) -> None: ...
def parse_hashmap(dict_cell: Slice, key_len: int) -> dict[str, Any] | None: ...
def parse_hashmap_aug(
    dict_cell: Slice,
    key_len: int,
    x_deserializer: Callable[..., Any],
    y_deserializer: Callable[..., Any],
) -> tuple[dict[int, Any], list[Any]] | None: ...



from pytoniq_core.tlb.utils import (
    HashUpdate,
    MerkleUpdate,
    deserialize_shard_hashes,
    uint64_to_int64,
)

__all__ = [
    "HashUpdate",
    "MerkleUpdate",
    "deserialize_shard_hashes",
    "uint64_to_int64",
]



from .tlb import (
    BaseDNSRecord,
    BaseDNSRecordAddress,
    BaseDNSRecordBinary,
    ChangeDNSRecordBody,
    DNSBalanceReleaseBody,
    DNSRecordDNSNextResolver,
    DNSRecords,
    DNSRecordSite,
    DNSRecordStorage,
    DNSRecordText,
    DNSRecordWallet,
    RenewDNSBody,
    TONDNSAuction,
    TONDNSCollectionData,
    TONDNSItemData,
)

__all__ = [
    "BaseDNSRecord",
    "BaseDNSRecordAddress",
    "BaseDNSRecordBinary",
    "ChangeDNSRecordBody",
    "DNSBalanceReleaseBody",
    "DNSRecordDNSNextResolver",
    "DNSRecordSite",
    "DNSRecordStorage",
    "DNSRecordText",
    "DNSRecordWallet",
    "DNSRecords",
    "RenewDNSBody",
    "TONDNSAuction",
    "TONDNSCollectionData",
    "TONDNSItemData",
]



class CellTypes:
    ordinary: int
    pruned_branch: int
    library_ref: int
    merkle_proof: int
    merkle_update: int

class LevelMask:
    def __init__(self, m: int) -> None: ...
    @property
    def mask(self) -> int: ...
    @property
    def level(self) -> int: ...
    @property
    def hash_index(self) -> int: ...
    def get_level(self) -> int: ...
    def get_hash_index(self) -> int: ...
    def apply(self, level: int) -> LevelMask: ...
    def is_significant(self, level: int) -> bool: ...



from typing import Any

class Crypto:
    magic: bytes
    magic_aes: bytes
    def __init__(self) -> None: ...
    def get_key_id(self) -> bytes: ...
    def get_aes_key_id(self) -> bytes: ...

class Server(Crypto):
    host: str
    port: int
    ed25519_public: Any
    x25519_public: Any
    def __init__(self, host: str, port: int, pub_key: bytes) -> None: ...

class Client(Crypto):
    ed25519_private: Any
    ed25519_public: Any
    x25519_private: Any
    x25519_public: Any
    def __init__(self, ed25519_private_key: bytes) -> None: ...
    @staticmethod
    def generate_ed25519_private_key() -> bytes: ...
    def sign(self, message: bytes) -> bytes: ...

class AdnlChannel:
    enc_key: bytes
    dec_key: bytes
    client_aes_key_id: bytes
    server_aes_key_id: bytes
    def __init__(self, client: Client, server: Server, local_id: bytes, peer_id: bytes) -> None: ...
    def encrypt(self, data: bytes) -> bytes: ...
    def decrypt(self, encrypted_data: bytes, checksum: bytes) -> bytes: ...

def aes_ctr_decrypt(cipher: Any, data: bytes) -> bytes: ...
def aes_ctr_encrypt(cipher: Any, data: bytes) -> bytes: ...
def create_aes_ctr_cipher(key: bytes, iv: bytes) -> Any: ...
def create_aes_ctr_sipher_from_key_n_data(key: bytes, data: bytes) -> Any: ...
def get_key_aes_id(key: bytes) -> bytes: ...
def get_random(bytes_size: int) -> bytes: ...
def get_shared_key(private_key: bytes, public_key: bytes) -> bytes: ...
def get_signature(private_key: Any, message: bytes) -> bytes: ...



from .boc import Address as Address
from .boc import AddressError as AddressError
from .boc import Anycast as Anycast
from .boc import BitarrayLike as BitarrayLike
from .boc import Builder as Builder
from .boc import BytesLike as BytesLike
from .boc import Cell as Cell
from .boc import CellError as CellError
from .boc import CellTypes as CellTypes
from .boc import DictError as DictError
from .boc import ExternalAddress as ExternalAddress
from .boc import HashMap as HashMap
from .boc import Key as Key
from .boc import LevelMask as LevelMask
from .boc import Slice as Slice
from .boc import SliceError as SliceError
from .boc import TvmBitarray as TvmBitarray
from .boc import TvmBitarrayException as TvmBitarrayException
from .boc import TvmBitarrayOverflowException as TvmBitarrayOverflowException
from .boc import TvmBitarrayUnderflowException as TvmBitarrayUnderflowException
from .boc import begin_cell as begin_cell
from .contrib.config import MAINNET_GLOBAL_CONFIG_URL as MAINNET_GLOBAL_CONFIG_URL
from .contrib.config import TESTNET_GLOBAL_CONFIG_URL as TESTNET_GLOBAL_CONFIG_URL
from .contrib.config import AdnlAddressConfig as AdnlAddressConfig
from .contrib.config import AdnlAddressListConfig as AdnlAddressListConfig
from .contrib.config import BlockRef as BlockRef
from .contrib.config import ConfigModel as ConfigModel
from .contrib.config import DhtConfig as DhtConfig
from .contrib.config import DhtNodeConfig as DhtNodeConfig
from .contrib.config import DhtNodeIDConfig as DhtNodeIDConfig
from .contrib.config import DhtNodesConfig as DhtNodesConfig
from .contrib.config import GlobalConfig as GlobalConfig
from .contrib.config import LiteServerConfig as LiteServerConfig
from .contrib.config import LiteServerIDConfig as LiteServerIDConfig
from .contrib.config import ValidatorConfig as ValidatorConfig
from .contrib.config import get_mainnet_global_config as get_mainnet_global_config
from .contrib.config import get_testnet_global_config as get_testnet_global_config
from .contrib.config import load_global_config as load_global_config
from .contrib.contracts.codes import CONTRACT_CODES as CONTRACT_CODES
from .contrib.contracts.dns.tlb import BaseDNSRecord as BaseDNSRecord
from .contrib.contracts.dns.tlb import BaseDNSRecordAddress as BaseDNSRecordAddress
from .contrib.contracts.dns.tlb import BaseDNSRecordBinary as BaseDNSRecordBinary
from .contrib.contracts.dns.tlb import ChangeDNSRecordBody as ChangeDNSRecordBody
from .contrib.contracts.dns.tlb import DNSBalanceReleaseBody as DNSBalanceReleaseBody
from .contrib.contracts.dns.tlb import DNSRecordDNSNextResolver as DNSRecordDNSNextResolver
from .contrib.contracts.dns.tlb import DNSRecords as DNSRecords
from .contrib.contracts.dns.tlb import DNSRecordSite as DNSRecordSite
from .contrib.contracts.dns.tlb import DNSRecordStorage as DNSRecordStorage
from .contrib.contracts.dns.tlb import DNSRecordText as DNSRecordText
from .contrib.contracts.dns.tlb import DNSRecordWallet as DNSRecordWallet
from .contrib.contracts.dns.tlb import RenewDNSBody as RenewDNSBody
from .contrib.contracts.dns.tlb import TONDNSAuction as TONDNSAuction
from .contrib.contracts.dns.tlb import TONDNSCollectionData as TONDNSCollectionData
from .contrib.contracts.dns.tlb import TONDNSItemData as TONDNSItemData
from .contrib.contracts.jetton.tlb import JettonBurnBody as JettonBurnBody
from .contrib.contracts.jetton.tlb import JettonChangeAdminBody as JettonChangeAdminBody
from .contrib.contracts.jetton.tlb import JettonChangeContentBody as JettonChangeContentBody
from .contrib.contracts.jetton.tlb import JettonClaimAdminBody as JettonClaimAdminBody
from .contrib.contracts.jetton.tlb import JettonDiscoveryBody as JettonDiscoveryBody
from .contrib.contracts.jetton.tlb import JettonDropAdminBody as JettonDropAdminBody
from .contrib.contracts.jetton.tlb import JettonInternalTransferBody as JettonInternalTransferBody
from .contrib.contracts.jetton.tlb import JettonMasterStablecoinData as JettonMasterStablecoinData
from .contrib.contracts.jetton.tlb import JettonMasterStandardData as JettonMasterStandardData
from .contrib.contracts.jetton.tlb import JettonMintBody as JettonMintBody
from .contrib.contracts.jetton.tlb import JettonStandardChangeAdminBody as JettonStandardChangeAdminBody
from .contrib.contracts.jetton.tlb import JettonStandardChangeContentBody as JettonStandardChangeContentBody
from .contrib.contracts.jetton.tlb import JettonStandardMintBody as JettonStandardMintBody
from .contrib.contracts.jetton.tlb import JettonTopUpBody as JettonTopUpBody
from .contrib.contracts.jetton.tlb import JettonTransferBody as JettonTransferBody
from .contrib.contracts.jetton.tlb import JettonUpgradeBody as JettonUpgradeBody
from .contrib.contracts.jetton.tlb import JettonWalletStablecoinData as JettonWalletStablecoinData
from .contrib.contracts.jetton.tlb import JettonWalletStablecoinV2Data as JettonWalletStablecoinV2Data
from .contrib.contracts.jetton.tlb import JettonWalletStandardData as JettonWalletStandardData
from .contrib.contracts.nft.tlb import NFTCollectionBatchMintItemBody as NFTCollectionBatchMintItemBody
from .contrib.contracts.nft.tlb import NFTCollectionChangeContentBody as NFTCollectionChangeContentBody
from .contrib.contracts.nft.tlb import NFTCollectionChangeOwnerBody as NFTCollectionChangeOwnerBody
from .contrib.contracts.nft.tlb import NFTCollectionContent as NFTCollectionContent
from .contrib.contracts.nft.tlb import NFTCollectionData as NFTCollectionData
from .contrib.contracts.nft.tlb import NFTCollectionMintItemBody as NFTCollectionMintItemBody
from .contrib.contracts.nft.tlb import NFTDestroyBody as NFTDestroyBody
from .contrib.contracts.nft.tlb import NFTEditContentBody as NFTEditContentBody
from .contrib.contracts.nft.tlb import NFTItemEditableData as NFTItemEditableData
from .contrib.contracts.nft.tlb import NFTItemEditableMintRef as NFTItemEditableMintRef
from .contrib.contracts.nft.tlb import NFTItemSoulboundData as NFTItemSoulboundData
from .contrib.contracts.nft.tlb import NFTItemSoulboundMintRef as NFTItemSoulboundMintRef
from .contrib.contracts.nft.tlb import NFTItemStandardData as NFTItemStandardData
from .contrib.contracts.nft.tlb import NFTItemStandardMintRef as NFTItemStandardMintRef
from .contrib.contracts.nft.tlb import NFTRevokeBody as NFTRevokeBody
from .contrib.contracts.nft.tlb import NFTTransferBody as NFTTransferBody
from .contrib.contracts.nft.tlb import NFTTransferEditorshipBody as NFTTransferEditorshipBody
from .contrib.contracts.nft.tlb import OffchainCommonContent as OffchainCommonContent
from .contrib.contracts.nft.tlb import OffchainContent as OffchainContent
from .contrib.contracts.nft.tlb import OffchainItemContent as OffchainItemContent
from .contrib.contracts.nft.tlb import OnchainContent as OnchainContent
from .contrib.contracts.nft.tlb import RoyaltyParams as RoyaltyParams
from .contrib.contracts.opcodes import OpCode as OpCode
from .contrib.contracts.telegram.tlb import TeleCollectionData as TeleCollectionData
from .contrib.contracts.telegram.tlb import TeleItemAuction as TeleItemAuction
from .contrib.contracts.telegram.tlb import TeleItemAuctionConfig as TeleItemAuctionConfig
from .contrib.contracts.telegram.tlb import TeleItemAuctionState as TeleItemAuctionState
from .contrib.contracts.telegram.tlb import TeleItemCancelAuctionBody as TeleItemCancelAuctionBody
from .contrib.contracts.telegram.tlb import TeleItemConfig as TeleItemConfig
from .contrib.contracts.telegram.tlb import TeleItemContent as TeleItemContent
from .contrib.contracts.telegram.tlb import TeleItemData as TeleItemData
from .contrib.contracts.telegram.tlb import TeleItemStartAuctionBody as TeleItemStartAuctionBody
from .contrib.contracts.telegram.tlb import TeleItemState as TeleItemState
from .contrib.contracts.telegram.tlb import TeleItemTokenInfo as TeleItemTokenInfo
from .contrib.contracts.vanity.tlb import VanityDeployBody as VanityDeployBody
from .contrib.contracts.versions import ContractVersion as ContractVersion
from .contrib.contracts.wallet.configs import BaseContractConfig as BaseContractConfig
from .contrib.contracts.wallet.configs import BaseWalletConfig as BaseWalletConfig
from .contrib.contracts.wallet.configs import WalletHighloadV2Config as WalletHighloadV2Config
from .contrib.contracts.wallet.configs import WalletHighloadV3Config as WalletHighloadV3Config
from .contrib.contracts.wallet.configs import WalletPreprocessedV2Config as WalletPreprocessedV2Config
from .contrib.contracts.wallet.configs import WalletV1Config as WalletV1Config
from .contrib.contracts.wallet.configs import WalletV2Config as WalletV2Config
from .contrib.contracts.wallet.configs import WalletV3Config as WalletV3Config
from .contrib.contracts.wallet.configs import WalletV4Config as WalletV4Config
from .contrib.contracts.wallet.configs import WalletV5BetaConfig as WalletV5BetaConfig
from .contrib.contracts.wallet.configs import WalletV5Config as WalletV5Config
from .contrib.contracts.wallet.params import BaseWalletParams as BaseWalletParams
from .contrib.contracts.wallet.params import WalletHighloadV2Params as WalletHighloadV2Params
from .contrib.contracts.wallet.params import WalletHighloadV3Params as WalletHighloadV3Params
from .contrib.contracts.wallet.params import WalletPreprocessedV2Params as WalletPreprocessedV2Params
from .contrib.contracts.wallet.params import WalletV1Params as WalletV1Params
from .contrib.contracts.wallet.params import WalletV2Params as WalletV2Params
from .contrib.contracts.wallet.params import WalletV3Params as WalletV3Params
from .contrib.contracts.wallet.params import WalletV4Params as WalletV4Params
from .contrib.contracts.wallet.params import WalletV5BetaParams as WalletV5BetaParams
from .contrib.contracts.wallet.params import WalletV5Params as WalletV5Params
from .contrib.contracts.wallet.tlb import BaseWalletData as BaseWalletData
from .contrib.contracts.wallet.tlb import EncryptedTextCommentBody as EncryptedTextCommentBody
from .contrib.contracts.wallet.tlb import OutActionSendMsg as OutActionSendMsg
from .contrib.contracts.wallet.tlb import TextCommentBody as TextCommentBody
from .contrib.contracts.wallet.tlb import WalletHighloadV2Data as WalletHighloadV2Data
from .contrib.contracts.wallet.tlb import WalletHighloadV3Data as WalletHighloadV3Data
from .contrib.contracts.wallet.tlb import WalletPreprocessedV2Data as WalletPreprocessedV2Data
from .contrib.contracts.wallet.tlb import WalletV1Data as WalletV1Data
from .contrib.contracts.wallet.tlb import WalletV2Data as WalletV2Data
from .contrib.contracts.wallet.tlb import WalletV3Data as WalletV3Data
from .contrib.contracts.wallet.tlb import WalletV4Data as WalletV4Data
from .contrib.contracts.wallet.tlb import WalletV5BetaData as WalletV5BetaData
from .contrib.contracts.wallet.tlb import WalletV5Data as WalletV5Data
from .contrib.contracts.wallet.tlb import WalletV5SubwalletID as WalletV5SubwalletID
from .contrib.messages import ExternalMessage as ExternalMessage
from .contrib.messages import InternalMessage as InternalMessage
from .contrib.messages import MessageAny as MessageAny
from .contrib.messages import WalletMessage as WalletMessage
from .contrib.types import ADNL as ADNL
from .contrib.types import DEFAULT_SENDMODE as DEFAULT_SENDMODE
from .contrib.types import DEFAULT_SUBWALLET_ID as DEFAULT_SUBWALLET_ID
from .contrib.types import MAINNET_GENESIS_UTIME as MAINNET_GENESIS_UTIME
from .contrib.types import MASTERCHAIN_SHARD as MASTERCHAIN_SHARD
from .contrib.types import AddressLike as AddressLike
from .contrib.types import BagID as BagID
from .contrib.types import Binary as Binary
from .contrib.types import BinaryLike as BinaryLike
from .contrib.types import ContractState as ContractState
from .contrib.types import DNSCategory as DNSCategory
from .contrib.types import DNSPrefix as DNSPrefix
from .contrib.types import MetadataPrefix as MetadataPrefix
from .contrib.types import NetworkGlobalID as NetworkGlobalID
from .contrib.types import PrivateKey as PrivateKey
from .contrib.types import PublicKey as PublicKey
from .contrib.types import SendMode as SendMode
from .contrib.types import SignatureDomain as SignatureDomain
from .contrib.types import WorkchainID as WorkchainID
from .contrib.utils import TextCipher as TextCipher
from .contrib.utils import calc_valid_until as calc_valid_until
from .contrib.utils import cell_hash as cell_hash
from .contrib.utils import cell_to_b64 as cell_to_b64
from .contrib.utils import cell_to_hex as cell_to_hex
from .contrib.utils import decode_dns_name as decode_dns_name
from .contrib.utils import encode_dns_name as encode_dns_name
from .contrib.utils import load_json as load_json
from .contrib.utils import maybe_stack_addr as maybe_stack_addr
from .contrib.utils import norm_stack_cell as norm_stack_cell
from .contrib.utils import norm_stack_num as norm_stack_num
from .contrib.utils import normalize_hash as normalize_hash
from .contrib.utils import parse_stack_config as parse_stack_config
from .contrib.utils import slice_hash as slice_hash
from .contrib.utils import string_hash as string_hash
from .contrib.utils import to_amount as to_amount
from .contrib.utils import to_cell as to_cell
from .contrib.utils import to_nano as to_nano
from .crypto import PBKDF_ITERATIONS as PBKDF_ITERATIONS
from .crypto import AdnlChannel as AdnlChannel
from .crypto import Client as Client
from .crypto import Crypto as Crypto
from .crypto import Server as Server
from .crypto import aes_ctr_decrypt as aes_ctr_decrypt
from .crypto import aes_ctr_encrypt as aes_ctr_encrypt
from .crypto import crc16 as crc16
from .crypto import crc32c as crc32c
from .crypto import create_aes_ctr_cipher as create_aes_ctr_cipher
from .crypto import create_aes_ctr_sipher_from_key_n_data as create_aes_ctr_sipher_from_key_n_data
from .crypto import get_key_aes_id as get_key_aes_id
from .crypto import get_random as get_random
from .crypto import get_secure_random_number as get_secure_random_number
from .crypto import get_shared_key as get_shared_key
from .crypto import get_signature as get_signature
from .crypto import is_basic_seed as is_basic_seed
from .crypto import mnemonic_is_valid as mnemonic_is_valid
from .crypto import mnemonic_new as mnemonic_new
from .crypto import mnemonic_to_entropy as mnemonic_to_entropy
from .crypto import mnemonic_to_private_key as mnemonic_to_private_key
from .crypto import mnemonic_to_seed as mnemonic_to_seed
from .crypto import mnemonic_to_wallet_key as mnemonic_to_wallet_key
from .crypto import private_key_to_public_key as private_key_to_public_key
from .crypto import sign_message as sign_message
from .crypto import verify_sign as verify_sign
from .crypto import words as words
from .proof import ProofError as ProofError
from .proof import calculate_node_id_short as calculate_node_id_short
from .proof import check_account_proof as check_account_proof
from .proof import check_block_header_proof as check_block_header_proof
from .proof import check_block_signatures as check_block_signatures
from .proof import check_proof as check_proof
from .proof import check_shard_proof as check_shard_proof
from .proof import compute_validator_set as compute_validator_set
from .tl import BlockId as BlockId
from .tl import BlockIdExt as BlockIdExt
from .tl import TlError as TlError
from .tl import TlGenerator as TlGenerator
from .tl import TlRegistrator as TlRegistrator
from .tl import TlSchema as TlSchema
from .tl import TlSchemas as TlSchemas
from .tlb import Account as Account
from .tlb import AccountBlock as AccountBlock
from .tlb import AccountError as AccountError
from .tlb import AccountState as AccountState
from .tlb import AccountStatus as AccountStatus
from .tlb import AccountStorage as AccountStorage
from .tlb import AccStatusChange as AccStatusChange
from .tlb import BinTree as BinTree
from .tlb import BlkMasterInfo as BlkMasterInfo
from .tlb import BlkPrevInfo as BlkPrevInfo
from .tlb import Block as Block
from .tlb import BlockCreateFees as BlockCreateFees
from .tlb import BlockCreateStats as BlockCreateStats
from .tlb import BlockError as BlockError
from .tlb import BlockExtra as BlockExtra
from .tlb import BlockInfo as BlockInfo
from .tlb import BlockLimits as BlockLimits
from .tlb import CatchainConfig as CatchainConfig
from .tlb import CommonMsgInfo as CommonMsgInfo
from .tlb import ComplaintPricing as ComplaintPricing
from .tlb import ComputeSkipReason as ComputeSkipReason
from .tlb import ConfigError as ConfigError
from .tlb import ConfigParam as ConfigParam
from .tlb import ConfigParam0 as ConfigParam0
from .tlb import ConfigParam1 as ConfigParam1
from .tlb import ConfigParam2 as ConfigParam2
from .tlb import ConfigParam3 as ConfigParam3
from .tlb import ConfigParam4 as ConfigParam4
from .tlb import ConfigParam5 as ConfigParam5
from .tlb import ConfigParam6 as ConfigParam6
from .tlb import ConfigParam7 as ConfigParam7
from .tlb import ConfigParam8 as ConfigParam8
from .tlb import ConfigParam9 as ConfigParam9
from .tlb import ConfigParam10 as ConfigParam10
from .tlb import ConfigParam11 as ConfigParam11
from .tlb import ConfigParam12 as ConfigParam12
from .tlb import ConfigParam13 as ConfigParam13
from .tlb import ConfigParam14 as ConfigParam14
from .tlb import ConfigParam15 as ConfigParam15
from .tlb import ConfigParam16 as ConfigParam16
from .tlb import ConfigParam17 as ConfigParam17
from .tlb import ConfigParam18 as ConfigParam18
from .tlb import ConfigParam20 as ConfigParam20
from .tlb import ConfigParam21 as ConfigParam21
from .tlb import ConfigParam22 as ConfigParam22
from .tlb import ConfigParam23 as ConfigParam23
from .tlb import ConfigParam24 as ConfigParam24
from .tlb import ConfigParam25 as ConfigParam25
from .tlb import ConfigParam28 as ConfigParam28
from .tlb import ConfigParam29 as ConfigParam29
from .tlb import ConfigParam31 as ConfigParam31
from .tlb import ConfigParam32 as ConfigParam32
from .tlb import ConfigParam33 as ConfigParam33
from .tlb import ConfigParam34 as ConfigParam34
from .tlb import ConfigParam35 as ConfigParam35
from .tlb import ConfigParam36 as ConfigParam36
from .tlb import ConfigParam37 as ConfigParam37
from .tlb import ConfigParam44 as ConfigParam44
from .tlb import ConfigParam71 as ConfigParam71
from .tlb import ConfigParam72 as ConfigParam72
from .tlb import ConfigParam73 as ConfigParam73
from .tlb import ConfigParam79 as ConfigParam79
from .tlb import ConfigParam81 as ConfigParam81
from .tlb import ConfigParam82 as ConfigParam82
from .tlb import ConfigParams as ConfigParams
from .tlb import ConfigProposalSetup as ConfigProposalSetup
from .tlb import ConfigVotingSetup as ConfigVotingSetup
from .tlb import ConsensusConfig as ConsensusConfig
from .tlb import Counters as Counters
from .tlb import CreatorStats as CreatorStats
from .tlb import CurrencyCollection as CurrencyCollection
from .tlb import DepthBalanceInfo as DepthBalanceInfo
from .tlb import ExtBlkRef as ExtBlkRef
from .tlb import ExternalMsgInfo as ExternalMsgInfo
from .tlb import ExternalOutMsgInfo as ExternalOutMsgInfo
from .tlb import ExtraCurrencyCollection as ExtraCurrencyCollection
from .tlb import FutureSplitMerge as FutureSplitMerge
from .tlb import GasLimitsPrices as GasLimitsPrices
from .tlb import GlobalVersion as GlobalVersion
from .tlb import HashUpdate as HashUpdate
from .tlb import ImportFees as ImportFees
from .tlb import InMsg as InMsg
from .tlb import IntermediateAddress as IntermediateAddress
from .tlb import InternalMsgInfo as InternalMsgInfo
from .tlb import JettonBridgeParams as JettonBridgeParams
from .tlb import JettonBridgePrices as JettonBridgePrices
from .tlb import KeyExtBlkRef as KeyExtBlkRef
from .tlb import KeyMaxLt as KeyMaxLt
from .tlb import LibRef as LibRef
from .tlb import McBlockExtra as McBlockExtra
from .tlb import McStateExtra as McStateExtra
from .tlb import MerkleUpdate as MerkleUpdate
from .tlb import MsgEnvelope as MsgEnvelope
from .tlb import MsgForwardPrices as MsgForwardPrices
from .tlb import MsgMetadata as MsgMetadata
from .tlb import OldMcBlocksInfo as OldMcBlocksInfo
from .tlb import OracleBridgeParams as OracleBridgeParams
from .tlb import OutAction as OutAction
from .tlb import OutList as OutList
from .tlb import OutMsg as OutMsg
from .tlb import OutMsgQueue as OutMsgQueue
from .tlb import OutMsgQueueInfo as OutMsgQueueInfo
from .tlb import ParamLimits as ParamLimits
from .tlb import ShardAccount as ShardAccount
from .tlb import ShardAccounts as ShardAccounts
from .tlb import ShardDescr as ShardDescr
from .tlb import ShardIdent as ShardIdent
from .tlb import ShardState as ShardState
from .tlb import ShardStateUnsplit as ShardStateUnsplit
from .tlb import SigPubKey as SigPubKey
from .tlb import SimpleAccount as SimpleAccount
from .tlb import SimpleAccountState as SimpleAccountState
from .tlb import SplitMergeInfo as SplitMergeInfo
from .tlb import StateInit as StateInit
from .tlb import StorageExtraInfo as StorageExtraInfo
from .tlb import StorageInfo as StorageInfo
from .tlb import StoragePrices as StoragePrices
from .tlb import StorageUsed as StorageUsed
from .tlb import StorageUsedShort as StorageUsedShort
from .tlb import SuspendedAddressList as SuspendedAddressList
from .tlb import TickTock as TickTock
from .tlb import TlbError as TlbError
from .tlb import TlbScheme as TlbScheme
from .tlb import TrActionPhase as TrActionPhase
from .tlb import Transaction as Transaction
from .tlb import TransactionDescr as TransactionDescr
from .tlb import TransactionError as TransactionError
from .tlb import TransactionMergeInstall as TransactionMergeInstall
from .tlb import TransactionMergePrepare as TransactionMergePrepare
from .tlb import TransactionOrdinary as TransactionOrdinary
from .tlb import TransactionSplitInstall as TransactionSplitInstall
from .tlb import TransactionSplitPrepare as TransactionSplitPrepare
from .tlb import TransactionStorage as TransactionStorage
from .tlb import TransactionTickTock as TransactionTickTock
from .tlb import TrBouncePhase as TrBouncePhase
from .tlb import TrComputePhase as TrComputePhase
from .tlb import TrCreditPhase as TrCreditPhase
from .tlb import TrStoragePhase as TrStoragePhase
from .tlb import ValidatorDescr as ValidatorDescr
from .tlb import ValidatorInfo as ValidatorInfo
from .tlb import ValidatorSet as ValidatorSet
from .tlb import ValueFlow as ValueFlow
from .tlb import VmCellSlice as VmCellSlice
from .tlb import VmCont as VmCont
from .tlb import VmControlData as VmControlData
from .tlb import VmError as VmError
from .tlb import VmSaveList as VmSaveList
from .tlb import VmStack as VmStack
from .tlb import VmStackList as VmStackList
from .tlb import VmStackValue as VmStackValue
from .tlb import VmTuple as VmTuple
from .tlb import VmTupleRef as VmTupleRef
from .tlb import WcSplitMergeTimings as WcSplitMergeTimings
from .tlb import WorkchainDescr as WorkchainDescr
from .tlb import WorkchainFormat as WorkchainFormat
from .tlb import deserialize_shard_hashes as deserialize_shard_hashes



from .check_proof import (
    ProofError,
    calculate_node_id_short,
    check_account_proof,
    check_block_header_proof,
    check_block_signatures,
    check_proof,
    check_shard_proof,
    compute_validator_set,
)

__all__ = [
    "ProofError",
    "calculate_node_id_short",
    "check_account_proof",
    "check_block_header_proof",
    "check_block_signatures",
    "check_proof",
    "check_shard_proof",
    "compute_validator_set",
]



from __future__ import annotations

import secrets
import time
from dataclasses import dataclass

from ton_core.contrib.contracts.opcodes import OpCode
from ton_core.contrib.types import DEFAULT_SENDMODE, SendMode

__all__ = [
    "BaseWalletParams",
    "WalletHighloadV2Params",
    "WalletHighloadV3Params",
    "WalletPreprocessedV2Params",
    "WalletV1Params",
    "WalletV2Params",
    "WalletV3Params",
    "WalletV4Params",
    "WalletV5BetaParams",
    "WalletV5Params",
]


@dataclass
class BaseWalletParams:
    """Base parameters for wallet transaction building."""


@dataclass
class WalletV1Params(BaseWalletParams):
    """Transaction parameters for Wallet v1."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""


@dataclass
class WalletV2Params(BaseWalletParams):
    """Transaction parameters for Wallet v2."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""


@dataclass
class WalletV3Params(BaseWalletParams):
    """Transaction parameters for Wallet v3."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""


@dataclass
class WalletV4Params(BaseWalletParams):
    """Transaction parameters for Wallet v4."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""
    op_code: int = 0x00
    """Operation code (0x00 for simple transfer)."""


@dataclass
class WalletV5BetaParams(BaseWalletParams):
    """Transaction parameters for Wallet v5 Beta."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""
    op_code: int = OpCode.AUTH_SIGNED_EXTERNAL
    """Operation code (default: 0x7369676E)."""


@dataclass
class WalletV5Params(BaseWalletParams):
    """Transaction parameters for Wallet v5."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""
    op_code: int = OpCode.AUTH_SIGNED_EXTERNAL
    """Operation code (default: 0x7369676E)."""


@dataclass
class WalletHighloadV2Params(BaseWalletParams):
    """Transaction parameters for Highload Wallet v2."""

    bounded_id: int | None = None
    """Bounded query ID combining TTL and query_id (auto-generated if None)."""
    query_id: int | None = None
    """Random 32-bit query identifier (auto-generated if None)."""
    message_ttl: int = 60 * 5
    """Message time-to-live in seconds (default: 300)."""

    def __post_init__(self) -> None:
        """Auto-generate query_id and bounded_id if not provided."""
        if self.query_id is None:
            self.query_id = secrets.randbits(32)
        if self.bounded_id is None:
            now = int(time.time())
            ttl_u32 = (now + self.message_ttl) & 0xFFFFFFFF
            qid_u32 = self.query_id & 0xFFFFFFFF
            self.bounded_id = (ttl_u32 << 32) | qid_u32


@dataclass
class WalletHighloadV3Params(BaseWalletParams):
    """Transaction parameters for Highload Wallet v3."""

    value_to_send: int | None = None
    """Total value in nanotons (calculated from messages if None)."""
    created_at: int | None = None
    """Creation unix timestamp (auto-generated if None)."""
    query_id: int | None = None
    """Query identifier derived from created_at (auto-generated if None)."""
    send_mode: SendMode | int = DEFAULT_SENDMODE
    """Message send mode flags."""

    def __post_init__(self) -> None:
        """Auto-generate created_at and query_id if not provided."""
        if self.created_at is None:
            self.created_at = int(time.time() - 60)
        if self.query_id is None:
            self.query_id = self.created_at % (1 << 23)


@dataclass
class WalletPreprocessedV2Params(BaseWalletParams):
    """Transaction parameters for Preprocessed Wallet v2."""

    seqno: int | None = None
    """Sequence number (fetched from contract if None)."""
    valid_until: int | None = None
    """Expiration unix timestamp, or None."""




from pytoniq_core.crypto.ciphers import (
    AdnlChannel,
    Client,
    Crypto,
    Server,
    aes_ctr_decrypt,
    aes_ctr_encrypt,
    create_aes_ctr_cipher,
    create_aes_ctr_sipher_from_key_n_data,
    get_key_aes_id,
    get_random,
    get_shared_key,
    get_signature,
)

__all__ = [
    "AdnlChannel",
    "Client",
    "Crypto",
    "Server",
    "aes_ctr_decrypt",
    "aes_ctr_encrypt",
    "create_aes_ctr_cipher",
    "create_aes_ctr_sipher_from_key_n_data",
    "get_key_aes_id",
    "get_random",
    "get_shared_key",
    "get_signature",
]



from .block import (
    BlockId as BlockId,
)
from .block import (
    BlockIdExt as BlockIdExt,
)
from .generator import (
    TlError as TlError,
)
from .generator import (
    TlGenerator as TlGenerator,
)
from .generator import (
    TlRegistrator as TlRegistrator,
)
from .generator import (
    TlSchema as TlSchema,
)
from .generator import (
    TlSchemas as TlSchemas,
)



from .account import Account as Account
from .account import AccountBlock as AccountBlock
from .account import AccountError as AccountError
from .account import AccountState as AccountState
from .account import AccountStatus as AccountStatus
from .account import AccountStorage as AccountStorage
from .account import ShardAccount as ShardAccount
from .account import SimpleAccount as SimpleAccount
from .account import SimpleAccountState as SimpleAccountState
from .account import StateInit as StateInit
from .account import StorageExtraInfo as StorageExtraInfo
from .account import StorageInfo as StorageInfo
from .account import StorageUsed as StorageUsed
from .account import StorageUsedShort as StorageUsedShort
from .account import TickTock as TickTock
from .block import BinTree as BinTree
from .block import BlkMasterInfo as BlkMasterInfo
from .block import BlkPrevInfo as BlkPrevInfo
from .block import Block as Block
from .block import BlockCreateStats as BlockCreateStats
from .block import BlockError as BlockError
from .block import BlockExtra as BlockExtra
from .block import BlockInfo as BlockInfo
from .block import ConfigParams as ConfigParams
from .block import Counters as Counters
from .block import CreatorStats as CreatorStats
from .block import CurrencyCollection as CurrencyCollection
from .block import DepthBalanceInfo as DepthBalanceInfo
from .block import ExtBlkRef as ExtBlkRef
from .block import ExtraCurrencyCollection as ExtraCurrencyCollection
from .block import FutureSplitMerge as FutureSplitMerge
from .block import GlobalVersion as GlobalVersion
from .block import KeyExtBlkRef as KeyExtBlkRef
from .block import KeyMaxLt as KeyMaxLt
from .block import McBlockExtra as McBlockExtra
from .block import McStateExtra as McStateExtra
from .block import OldMcBlocksInfo as OldMcBlocksInfo
from .block import OutMsgQueue as OutMsgQueue
from .block import OutMsgQueueInfo as OutMsgQueueInfo
from .block import ShardAccounts as ShardAccounts
from .block import ShardDescr as ShardDescr
from .block import ShardIdent as ShardIdent
from .block import ShardState as ShardState
from .block import ShardStateUnsplit as ShardStateUnsplit
from .block import ValidatorInfo as ValidatorInfo
from .block import ValueFlow as ValueFlow
from .config import BlockCreateFees as BlockCreateFees
from .config import BlockLimits as BlockLimits
from .config import CatchainConfig as CatchainConfig
from .config import ComplaintPricing as ComplaintPricing
from .config import ConfigError as ConfigError
from .config import ConfigParam as ConfigParam
from .config import ConfigParam0 as ConfigParam0
from .config import ConfigParam1 as ConfigParam1
from .config import ConfigParam2 as ConfigParam2
from .config import ConfigParam3 as ConfigParam3
from .config import ConfigParam4 as ConfigParam4
from .config import ConfigParam5 as ConfigParam5
from .config import ConfigParam6 as ConfigParam6
from .config import ConfigParam7 as ConfigParam7
from .config import ConfigParam8 as ConfigParam8
from .config import ConfigParam9 as ConfigParam9
from .config import ConfigParam10 as ConfigParam10
from .config import ConfigParam11 as ConfigParam11
from .config import ConfigParam12 as ConfigParam12
from .config import ConfigParam13 as ConfigParam13
from .config import ConfigParam14 as ConfigParam14
from .config import ConfigParam15 as ConfigParam15
from .config import ConfigParam16 as ConfigParam16
from .config import ConfigParam17 as ConfigParam17
from .config import ConfigParam18 as ConfigParam18
from .config import ConfigParam20 as ConfigParam20
from .config import ConfigParam21 as ConfigParam21
from .config import ConfigParam22 as ConfigParam22
from .config import ConfigParam23 as ConfigParam23
from .config import ConfigParam24 as ConfigParam24
from .config import ConfigParam25 as ConfigParam25
from .config import ConfigParam28 as ConfigParam28
from .config import ConfigParam29 as ConfigParam29
from .config import ConfigParam31 as ConfigParam31
from .config import ConfigParam32 as ConfigParam32
from .config import ConfigParam33 as ConfigParam33
from .config import ConfigParam34 as ConfigParam34
from .config import ConfigParam35 as ConfigParam35
from .config import ConfigParam36 as ConfigParam36
from .config import ConfigParam37 as ConfigParam37
from .config import ConfigParam44 as ConfigParam44
from .config import ConfigParam71 as ConfigParam71
from .config import ConfigParam72 as ConfigParam72
from .config import ConfigParam73 as ConfigParam73
from .config import ConfigParam79 as ConfigParam79
from .config import ConfigParam81 as ConfigParam81
from .config import ConfigParam82 as ConfigParam82
from .config import ConfigProposalSetup as ConfigProposalSetup
from .config import ConfigVotingSetup as ConfigVotingSetup
from .config import ConsensusConfig as ConsensusConfig
from .config import GasLimitsPrices as GasLimitsPrices
from .config import JettonBridgeParams as JettonBridgeParams
from .config import JettonBridgePrices as JettonBridgePrices
from .config import MsgForwardPrices as MsgForwardPrices
from .config import OracleBridgeParams as OracleBridgeParams
from .config import ParamLimits as ParamLimits
from .config import SigPubKey as SigPubKey
from .config import StoragePrices as StoragePrices
from .config import SuspendedAddressList as SuspendedAddressList
from .config import ValidatorDescr as ValidatorDescr
from .config import ValidatorSet as ValidatorSet
from .config import WcSplitMergeTimings as WcSplitMergeTimings
from .config import WorkchainDescr as WorkchainDescr
from .config import WorkchainFormat as WorkchainFormat
from .tlb import TlbError as TlbError
from .tlb import TlbScheme as TlbScheme
from .transaction import AccStatusChange as AccStatusChange
from .transaction import CommonMsgInfo as CommonMsgInfo
from .transaction import ComputeSkipReason as ComputeSkipReason
from .transaction import ExternalMsgInfo as ExternalMsgInfo
from .transaction import ExternalOutMsgInfo as ExternalOutMsgInfo
from .transaction import ImportFees as ImportFees
from .transaction import InMsg as InMsg
from .transaction import IntermediateAddress as IntermediateAddress
from .transaction import InternalMsgInfo as InternalMsgInfo
from .transaction import LibRef as LibRef
from .transaction import MessageAny as MessageAny
from .transaction import MsgEnvelope as MsgEnvelope
from .transaction import MsgMetadata as MsgMetadata
from .transaction import OutAction as OutAction
from .transaction import OutList as OutList
from .transaction import OutMsg as OutMsg
from .transaction import SplitMergeInfo as SplitMergeInfo
from .transaction import TrActionPhase as TrActionPhase
from .transaction import Transaction as Transaction
from .transaction import TransactionDescr as TransactionDescr
from .transaction import TransactionError as TransactionError
from .transaction import TransactionMergeInstall as TransactionMergeInstall
from .transaction import TransactionMergePrepare as TransactionMergePrepare
from .transaction import TransactionOrdinary as TransactionOrdinary
from .transaction import TransactionSplitInstall as TransactionSplitInstall
from .transaction import TransactionSplitPrepare as TransactionSplitPrepare
from .transaction import TransactionStorage as TransactionStorage
from .transaction import TransactionTickTock as TransactionTickTock
from .transaction import TrBouncePhase as TrBouncePhase
from .transaction import TrComputePhase as TrComputePhase
from .transaction import TrCreditPhase as TrCreditPhase
from .transaction import TrStoragePhase as TrStoragePhase
from .utils import HashUpdate as HashUpdate
from .utils import MerkleUpdate as MerkleUpdate
from .utils import deserialize_shard_hashes as deserialize_shard_hashes
from .vm_stack import VmCellSlice as VmCellSlice
from .vm_stack import VmCont as VmCont
from .vm_stack import VmControlData as VmControlData
from .vm_stack import VmError as VmError
from .vm_stack import VmSaveList as VmSaveList
from .vm_stack import VmStack as VmStack
from .vm_stack import VmStackList as VmStackList
from .vm_stack import VmStackValue as VmStackValue
from .vm_stack import VmTuple as VmTuple
from .vm_stack import VmTupleRef as VmTupleRef



from __future__ import annotations

from ton_core.boc.address import Address
from ton_core.boc.builder import Builder
from ton_core.boc.cell import Cell
from ton_core.boc.slice import Slice
from ton_core.contrib.utils import cell_to_b64, cell_to_hex, normalize_hash
from ton_core.tlb.account import StateInit
from ton_core.tlb.block import CurrencyCollection
from ton_core.tlb.tlb import TlbScheme
from ton_core.tlb.transaction import (
    ExternalMsgInfo,
    InternalMsgInfo,
)
from ton_core.tlb.transaction import (
    MessageAny as MessageAnyBase,
)

__all__ = [
    "ExternalMessage",
    "InternalMessage",
    "MessageAny",
    "WalletMessage",
]


class WalletMessage(TlbScheme):
    """Wallet message wrapping send mode and a ``MessageAny`` cell reference."""

    send_mode: int
    """Send mode flags encoded as an 8-bit unsigned integer."""

    message: MessageAnyBase
    """The message payload wrapped as a ``MessageAny`` reference."""

    def __init__(self, send_mode: int, message: MessageAnyBase) -> None:
        """Initialize a wallet message.

        :param send_mode: Send mode flags encoded as an 8-bit unsigned integer.
        :param message: The message payload wrapped as a ``MessageAny`` reference.
        """
        self.send_mode = send_mode
        self.message = message

    def serialize(self) -> Cell:
        """Serialize to ``Cell``."""
        builder = Builder()
        builder.store_uint(self.send_mode, 8)
        builder.store_ref(self.message.serialize())
        return builder.end_cell()

    @classmethod
    def deserialize(cls, cell_slice: Slice) -> WalletMessage:
        """Deserialize from ``Slice``."""
        send_mode = cell_slice.load_uint(8)
        message = MessageAnyBase.deserialize(cell_slice.load_ref().begin_parse())
        return cls(send_mode=send_mode, message=message)


class MessageAny(MessageAnyBase):
    """Base class for TON messages with serialization helpers."""

    def to_cell(self) -> Cell:
        """Serialize to ``Cell``."""
        return self.serialize()

    def to_boc(self) -> bytes:
        """Serialize to BoC bytes."""
        return self.to_cell().to_boc()

    @property
    def as_hex(self) -> str:
        """Hex-encoded BoC string."""
        return cell_to_hex(self.to_cell())

    @property
    def as_b64(self) -> str:
        """Base64-encoded BoC string."""
        return cell_to_b64(self.to_cell())

    @property
    def normalized_hash(self) -> str:
        """Normalized message hash as hex string."""
        return normalize_hash(self)


class ExternalMessage(MessageAny):
    """External message for sending transactions to the TON blockchain."""

    def __init__(
        self,
        src: Address | None = None,
        dest: Address | None = None,
        import_fee: int = 0,
        body: Cell | None = None,
        state_init: StateInit | None = None,
    ) -> None:
        """Initialize an external message.

        :param src: Source address, or ``None``.
        :param dest: Destination contract address, or ``None``.
        :param import_fee: Import fee in nanotons.
        :param body: Signed message body cell, or ``None``.
        :param state_init: ``StateInit`` for deployment, or ``None``.
        """
        info = ExternalMsgInfo(src, dest, import_fee)
        super().__init__(info, state_init, body or Cell.empty())


class InternalMessage(MessageAny):
    """Internal message for on-chain contract-to-contract communication."""

    def __init__(
        self,
        ihr_disabled: bool | None = True,
        bounce: bool | None = None,
        bounced: bool | None = False,
        src: Address | str | None = None,
        dest: Address | str | None = None,
        value: CurrencyCollection | int = 0,
        ihr_fee: int = 0,
        fwd_fee: int = 0,
        created_lt: int = 0,
        created_at: int = 0,
        body: Cell | None = None,
        state_init: StateInit | None = None,
    ) -> None:
        """Initialize an internal message.

        :param ihr_disabled: Disable instant hypercube routing.
        :param bounce: Bounce on error, or ``None`` for auto-detect.
        :param bounced: Whether this is a bounced message.
        :param src: Source address, or ``None``.
        :param dest: Destination address, or ``None``.
        :param value: Amount in nanotons or ``CurrencyCollection``.
        :param ihr_fee: IHR fee in nanotons.
        :param fwd_fee: Forward fee in nanotons.
        :param created_lt: Logical time when created.
        :param created_at: Unix timestamp when created.
        :param body: Message body cell, or ``None``.
        :param state_init: ``StateInit`` for deployment, or ``None``.
        """
        if isinstance(src, str):
            src = Address(src)
        if isinstance(dest, str):
            dest = Address(dest)
        if bounce is None:
            bounce = dest.is_bounceable if dest and isinstance(dest, Address) else False
        if body is None:
            body = Cell.empty()
        if isinstance(value, int):
            value = CurrencyCollection(value)

        info = InternalMsgInfo(
            ihr_disabled=ihr_disabled,
            bounce=bounce,
            bounced=bounced,
            src=src,
            dest=dest,
            value=value,
            ihr_fee=ihr_fee,
            fwd_fee=fwd_fee,
            created_lt=created_lt,
            created_at=created_at,
        )
        super().__init__(info, state_init, body)




from typing import Any

class TlbError(Exception): ...

class TlbScheme:
    def serialize(self, *args: Any, **kwargs: Any) -> Any: ...
    @classmethod
    def deserialize(cls, *args: Any, **kwargs: Any) -> Any: ...



from .ciphers import (
    AdnlChannel,
    Client,
    Crypto,
    Server,
    aes_ctr_decrypt,
    aes_ctr_encrypt,
    create_aes_ctr_cipher,
    create_aes_ctr_sipher_from_key_n_data,
    get_key_aes_id,
    get_random,
    get_shared_key,
    get_signature,
)
from .crc import (
    crc16,
    crc32c,
)
from .keys import (
    PBKDF_ITERATIONS,
    get_secure_random_number,
    is_basic_seed,
    mnemonic_is_valid,
    mnemonic_new,
    mnemonic_to_entropy,
    mnemonic_to_private_key,
    mnemonic_to_seed,
    mnemonic_to_wallet_key,
    private_key_to_public_key,
    words,
)
from .signature import (
    sign_message,
    verify_sign,
)

__all__ = [
    "PBKDF_ITERATIONS",
    "AdnlChannel",
    "Client",
    "Crypto",
    "Server",
    "aes_ctr_decrypt",
    "aes_ctr_encrypt",
    "crc16",
    "crc32c",
    "create_aes_ctr_cipher",
    "create_aes_ctr_sipher_from_key_n_data",
    "get_key_aes_id",
    "get_random",
    "get_secure_random_number",
    "get_shared_key",
    "get_signature",
    "is_basic_seed",
    "mnemonic_is_valid",
    "mnemonic_new",
    "mnemonic_to_entropy",
    "mnemonic_to_private_key",
    "mnemonic_to_seed",
    "mnemonic_to_wallet_key",
    "private_key_to_public_key",
    "sign_message",
    "verify_sign",
    "words",
]



from __future__ import annotations

from ton_core.boc import begin_cell
from ton_core.boc.cell import Cell
from ton_core.boc.slice import Slice
from ton_core.tlb.tlb import TlbScheme

__all__ = [
    "VanityDeployBody",
]


class VanityDeployBody(TlbScheme):
    """Message body for deploying contracts via Vanity."""

    def __init__(self, code: Cell, data: Cell) -> None:
        """Initialize VanityDeployBody.

        :param code: Contract code cell.
        :param data: Contract initial data cell.
        """
        self.code = code
        self.data = data

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_ref(self.code)
        cell.store_ref(self.data)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> VanityDeployBody:
        """Deserialize from Slice."""
        return cls(code=cs.load_ref(), data=cs.load_ref())



from pytoniq_core.tlb.account import (
    Account,
    AccountBlock,
    AccountError,
    AccountState,
    AccountStatus,
    AccountStorage,
    ShardAccount,
    SimpleAccount,
    SimpleAccountState,
    StateInit,
    StorageExtraInfo,
    StorageInfo,
    StorageUsed,
    StorageUsedShort,
    TickTock,
)

__all__ = [
    "Account",
    "AccountBlock",
    "AccountError",
    "AccountState",
    "AccountStatus",
    "AccountStorage",
    "ShardAccount",
    "SimpleAccount",
    "SimpleAccountState",
    "StateInit",
    "StorageExtraInfo",
    "StorageInfo",
    "StorageUsed",
    "StorageUsedShort",
    "TickTock",
]



from __future__ import annotations

from ton_core.boc import begin_cell
from ton_core.boc.cell import Cell
from ton_core.boc.slice import Slice
from ton_core.contrib.contracts._utils import _load_std_address
from ton_core.contrib.contracts.nft.tlb import OffchainContent, OnchainContent
from ton_core.contrib.contracts.opcodes import OpCode
from ton_core.contrib.types import AddressLike
from ton_core.tlb.tlb import TlbScheme

__all__ = [
    "JettonBurnBody",
    "JettonChangeAdminBody",
    "JettonChangeContentBody",
    "JettonClaimAdminBody",
    "JettonDiscoveryBody",
    "JettonDropAdminBody",
    "JettonInternalTransferBody",
    "JettonMasterStablecoinData",
    "JettonMasterStandardData",
    "JettonMintBody",
    "JettonStandardChangeAdminBody",
    "JettonStandardChangeContentBody",
    "JettonStandardMintBody",
    "JettonTopUpBody",
    "JettonTransferBody",
    "JettonUpgradeBody",
    "JettonWalletStablecoinData",
    "JettonWalletStablecoinV2Data",
    "JettonWalletStandardData",
]


class JettonMasterStandardData(TlbScheme):
    """On-chain data for standard Jetton minter contracts (TEP-74)."""

    def __init__(
        self,
        admin_address: AddressLike | None,
        content: OnchainContent | OffchainContent,
        jetton_wallet_code: Cell,
        total_supply: int = 0,
    ) -> None:
        """Initialize JettonMasterStandardData.

        :param admin_address: Admin address with minting rights.
        :param content: Jetton metadata (on-chain or off-chain).
        :param jetton_wallet_code: Code cell for Jetton wallet contracts.
        :param total_supply: Total minted supply in base units.
        """
        self.total_supply = total_supply
        self.admin_address = admin_address
        self.content = content
        self.jetton_wallet_code = jetton_wallet_code

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_coins(self.total_supply)
        cell.store_address(self.admin_address)
        cell.store_ref(self.content.serialize(True))
        cell.store_ref(self.jetton_wallet_code)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonMasterStandardData:
        """Deserialize from Slice."""
        from ton_core.contrib.types import MetadataPrefix

        total_supply = cs.load_coins() or 0
        admin_address = _load_std_address(cs)
        content = cs.load_ref().begin_parse()
        return cls(
            total_supply=total_supply,
            admin_address=admin_address,
            content=(
                OnchainContent.deserialize(content, False)
                if MetadataPrefix(content.load_uint(8)) == MetadataPrefix.ONCHAIN
                else OffchainContent.deserialize(content, False)
            ),
            jetton_wallet_code=cs.load_ref(),
        )


class JettonMasterStablecoinData(TlbScheme):
    """On-chain data for stablecoin Jetton minter contracts."""

    def __init__(
        self,
        admin_address: AddressLike | None,
        jetton_wallet_code: Cell,
        content: OffchainContent,
        next_admin_address: AddressLike | None = None,
        total_supply: int = 0,
    ) -> None:
        """Initialize JettonMasterStablecoinData.

        :param admin_address: Current admin address.
        :param jetton_wallet_code: Code cell for Jetton wallet contracts.
        :param content: Off-chain Jetton metadata.
        :param next_admin_address: Pending admin address, or None.
        :param total_supply: Total minted supply in base units.
        """
        self.total_supply = total_supply
        self.admin_address = admin_address
        self.next_admin_address = next_admin_address
        self.jetton_wallet_code = jetton_wallet_code
        self.content = content

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_coins(self.total_supply)
        cell.store_address(self.admin_address)
        cell.store_address(self.next_admin_address)
        cell.store_ref(self.jetton_wallet_code)
        cell.store_ref(self.content.serialize(False))
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonMasterStablecoinData:
        """Deserialize from Slice."""
        return cls(
            total_supply=cs.load_coins() or 0,
            admin_address=_load_std_address(cs),
            next_admin_address=_load_std_address(cs),
            jetton_wallet_code=cs.load_ref(),
            content=OffchainContent.deserialize(cs.load_ref().begin_parse(), False),
        )


class JettonWalletStandardData(TlbScheme):
    """On-chain data for standard Jetton wallet contracts (TEP-74)."""

    def __init__(
        self,
        owner_address: AddressLike | None,
        jetton_master_address: AddressLike | None,
        jetton_wallet_code: Cell,
        balance: int = 0,
    ) -> None:
        """Initialize JettonWalletStandardData.

        :param owner_address: Wallet owner address.
        :param jetton_master_address: Jetton minter address.
        :param jetton_wallet_code: Code cell for Jetton wallet contracts.
        :param balance: Current Jetton balance in base units.
        """
        self.balance = balance
        self.owner_address = owner_address
        self.jetton_master_address = jetton_master_address
        self.jetton_wallet_code = jetton_wallet_code

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_coins(self.balance)
        cell.store_address(self.owner_address)
        cell.store_address(self.jetton_master_address)
        cell.store_ref(self.jetton_wallet_code)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonWalletStandardData:
        """Deserialize from Slice."""
        return cls(
            balance=cs.load_coins() or 0,
            owner_address=_load_std_address(cs),
            jetton_master_address=_load_std_address(cs),
            jetton_wallet_code=cs.load_ref(),
        )


class JettonWalletStablecoinData(TlbScheme):
    """On-chain data for stablecoin Jetton wallet contracts."""

    def __init__(
        self,
        owner_address: AddressLike | None,
        jetton_master_address: AddressLike | None,
        status: int,
        balance: int = 0,
    ) -> None:
        """Initialize JettonWalletStablecoinData.

        :param owner_address: Wallet owner address.
        :param jetton_master_address: Jetton minter address.
        :param status: Wallet status flags (4 bits).
        :param balance: Current Jetton balance in base units.
        """
        self.status = status
        self.balance = balance
        self.owner_address = owner_address
        self.jetton_master_address = jetton_master_address

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(self.status, 4)
        cell.store_coins(self.balance)
        cell.store_address(self.owner_address)
        cell.store_address(self.jetton_master_address)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonWalletStablecoinData:
        """Deserialize from Slice."""
        return cls(
            status=cs.load_uint(4),
            balance=cs.load_coins() or 0,
            owner_address=_load_std_address(cs),
            jetton_master_address=_load_std_address(cs),
        )


class JettonWalletStablecoinV2Data(TlbScheme):
    """On-chain data for stablecoin v2 Jetton wallet contracts."""

    def __init__(
        self,
        owner_address: AddressLike | None,
        jetton_master_address: AddressLike | None,
        balance: int = 0,
    ) -> None:
        """Initialize JettonWalletStablecoinV2Data.

        :param owner_address: Wallet owner address.
        :param jetton_master_address: Jetton minter address.
        :param balance: Current Jetton balance in base units.
        """
        self.balance = balance
        self.owner_address = owner_address
        self.jetton_master_address = jetton_master_address

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_coins(self.balance)
        cell.store_address(self.owner_address)
        cell.store_address(self.jetton_master_address)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonWalletStablecoinV2Data:
        """Deserialize from Slice."""
        return cls(
            balance=cs.load_coins() or 0,
            owner_address=_load_std_address(cs),
            jetton_master_address=_load_std_address(cs),
        )


class JettonTopUpBody(TlbScheme):
    """Message body for topping up Jetton wallet balance."""

    def __init__(self, query_id: int = 0) -> None:
        """Initialize JettonTopUpBody.

        :param query_id: Query identifier.
        """
        self.query_id = query_id

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.TOP_UP, 32)
        cell.store_uint(self.query_id, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonTopUpBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(query_id=cs.load_uint(64))


class JettonInternalTransferBody(TlbScheme):
    """Message body for internal Jetton transfers between wallets."""

    def __init__(
        self,
        jetton_amount: int,
        forward_amount: int,
        from_address: AddressLike | None = None,
        response_address: AddressLike | None = None,
        forward_payload: Cell | None = None,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonInternalTransferBody.

        :param jetton_amount: Amount to transfer in base units.
        :param forward_amount: Amount to forward in nanotons.
        :param from_address: Original sender address, or None.
        :param response_address: Address for excess funds, or None.
        :param forward_payload: Payload to forward, or None.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.jetton_amount = jetton_amount
        self.from_address = from_address
        self.response_address = response_address
        self.forward_amount = forward_amount
        self.forward_payload = forward_payload

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_INTERNAL_TRANSFER, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_coins(self.jetton_amount)
        cell.store_address(self.from_address)
        cell.store_address(self.response_address)
        cell.store_coins(self.forward_amount)
        cell.store_maybe_ref(self.forward_payload)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonInternalTransferBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            jetton_amount=cs.load_coins() or 0,
            from_address=_load_std_address(cs),
            response_address=_load_std_address(cs),
            forward_amount=cs.load_coins() or 0,
            forward_payload=cs.load_maybe_ref(),
        )


class JettonTransferBody(TlbScheme):
    """Message body for Jetton transfers (TEP-74)."""

    def __init__(
        self,
        destination: AddressLike | None,
        jetton_amount: int,
        response_address: AddressLike | None = None,
        custom_payload: Cell | None = None,
        forward_payload: Cell | None = None,
        forward_amount: int = 1,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonTransferBody.

        :param destination: Recipient address.
        :param jetton_amount: Amount to transfer in base units.
        :param response_address: Address for excess funds, or None.
        :param custom_payload: Custom payload cell, or None.
        :param forward_payload: Payload to forward, or None.
        :param forward_amount: Amount to forward in nanotons.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.jetton_amount = jetton_amount
        self.destination = destination
        self.response_address = response_address
        self.custom_payload = custom_payload
        self.forward_amount = forward_amount
        self.forward_payload = forward_payload

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_TRANSFER, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_coins(self.jetton_amount)
        cell.store_address(self.destination)
        cell.store_address(self.response_address)
        cell.store_maybe_ref(self.custom_payload)
        cell.store_coins(self.forward_amount)
        cell.store_maybe_ref(self.forward_payload)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonTransferBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            jetton_amount=cs.load_coins() or 0,
            destination=_load_std_address(cs),
            response_address=_load_std_address(cs),
            custom_payload=cs.load_maybe_ref(),
            forward_amount=cs.load_coins() or 0,
            forward_payload=cs.load_maybe_ref(),
        )


class JettonMintBody(TlbScheme):
    """Message body for minting Jettons (stablecoin version)."""

    def __init__(
        self,
        destination: AddressLike | None,
        internal_transfer: JettonInternalTransferBody,
        forward_amount: int,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonMintBody.

        :param destination: Recipient wallet address.
        :param internal_transfer: Internal transfer body with mint details.
        :param forward_amount: Amount to forward in nanotons.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.destination = destination
        self.forward_amount = forward_amount
        self.internal_transfer = internal_transfer

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_MINT, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_address(self.destination)
        cell.store_coins(self.forward_amount)
        cell.store_ref(self.internal_transfer.serialize())
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonMintBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            destination=_load_std_address(cs),
            forward_amount=cs.load_coins() or 0,
            internal_transfer=JettonInternalTransferBody.deserialize(cs.load_ref().begin_parse()),
        )


class JettonStandardMintBody(TlbScheme):
    """Message body for minting Jettons (standard version)."""

    def __init__(
        self,
        destination: AddressLike | None,
        internal_transfer: JettonInternalTransferBody,
        forward_amount: int,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonStandardMintBody.

        :param destination: Recipient wallet address.
        :param internal_transfer: Internal transfer body with mint details.
        :param forward_amount: Amount to forward in nanotons.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.destination = destination
        self.forward_amount = forward_amount
        self.internal_transfer = internal_transfer

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(21, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_address(self.destination)
        cell.store_coins(self.forward_amount)
        cell.store_ref(self.internal_transfer.serialize())
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonStandardMintBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            destination=_load_std_address(cs),
            forward_amount=cs.load_coins() or 0,
            internal_transfer=JettonInternalTransferBody.deserialize(cs.load_ref().begin_parse()),
        )


class JettonChangeAdminBody(TlbScheme):
    """Message body for changing Jetton minter admin (stablecoin version)."""

    def __init__(
        self,
        admin_address: AddressLike | None,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonChangeAdminBody.

        :param admin_address: New admin address.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.admin_address = admin_address

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_CHANGE_ADMIN, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_address(self.admin_address)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonChangeAdminBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            admin_address=_load_std_address(cs),
        )


class JettonStandardChangeAdminBody(TlbScheme):
    """Message body for changing Jetton minter admin (standard version)."""

    def __init__(
        self,
        admin_address: AddressLike | None,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonStandardChangeAdminBody.

        :param admin_address: New admin address.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.admin_address = admin_address

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(3, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_address(self.admin_address)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonStandardChangeAdminBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            admin_address=_load_std_address(cs),
        )


class JettonDiscoveryBody(TlbScheme):
    """Message body for discovering Jetton wallet address."""

    def __init__(
        self,
        owner_address: AddressLike | None,
        include_address: bool = True,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonDiscoveryBody.

        :param owner_address: Owner address to query wallet for.
        :param include_address: Include address in response.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.owner_address = owner_address
        self.include_address = include_address

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_PROVIDE_WALLET_ADDRESS, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_address(self.owner_address)
        cell.store_bool(self.include_address)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonDiscoveryBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            owner_address=_load_std_address(cs),
            include_address=cs.load_bool(),
        )


class JettonClaimAdminBody(TlbScheme):
    """Message body for claiming Jetton minter admin rights."""

    def __init__(self, query_id: int = 0) -> None:
        """Initialize JettonClaimAdminBody.

        :param query_id: Query identifier.
        """
        self.query_id = query_id

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_CLAIM_ADMIN, 32)
        cell.store_uint(self.query_id, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonClaimAdminBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(query_id=cs.load_uint(64))


class JettonDropAdminBody(TlbScheme):
    """Message body for dropping Jetton minter admin rights."""

    def __init__(self, query_id: int = 0) -> None:
        """Initialize JettonDropAdminBody.

        :param query_id: Query identifier.
        """
        self.query_id = query_id

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_DROP_ADMIN, 32)
        cell.store_uint(self.query_id, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonDropAdminBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(query_id=cs.load_uint(64))


class JettonChangeContentBody(TlbScheme):
    """Message body for changing Jetton metadata (stablecoin version)."""

    def __init__(
        self,
        content: OffchainContent,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonChangeContentBody.

        :param content: New off-chain content.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.content = content

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_CHANGE_METADATA, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_snake_string(self.content.uri)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonChangeContentBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            content=OffchainContent(uri=cs.load_snake_string()),
        )


class JettonStandardChangeContentBody(TlbScheme):
    """Message body for changing Jetton metadata (standard version)."""

    def __init__(
        self,
        content: OnchainContent | OffchainContent,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonStandardChangeContentBody.

        :param content: New content (on-chain or off-chain).
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.content = content

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(4, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_ref(self.content.serialize(True))
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonStandardChangeContentBody:
        """Deserialize from Slice."""
        from ton_core.contrib.types import MetadataPrefix

        cs.skip_bits(32)
        query_id = cs.load_uint(64)
        content_cs = cs.load_ref().begin_parse()
        prefix = MetadataPrefix(content_cs.load_uint(8))
        content: OnchainContent | OffchainContent
        if prefix == MetadataPrefix.ONCHAIN:
            content = OnchainContent.deserialize(content_cs, False)
        else:
            content = OffchainContent.deserialize(content_cs, False)
        return cls(query_id=query_id, content=content)


class JettonBurnBody(TlbScheme):
    """Message body for burning Jettons (TEP-74)."""

    def __init__(
        self,
        jetton_amount: int,
        response_address: AddressLike | None,
        custom_payload: Cell | None = None,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonBurnBody.

        :param jetton_amount: Amount to burn in base units.
        :param response_address: Address for excess funds.
        :param custom_payload: Custom payload cell, or None.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.jetton_amount = jetton_amount
        self.response_address = response_address
        self.custom_payload = custom_payload

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_BURN, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_coins(self.jetton_amount)
        cell.store_address(self.response_address)
        cell.store_maybe_ref(self.custom_payload)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonBurnBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            jetton_amount=cs.load_coins() or 0,
            response_address=_load_std_address(cs),
            custom_payload=cs.load_maybe_ref(),
        )


class JettonUpgradeBody(TlbScheme):
    """Message body for upgrading Jetton contract code."""

    def __init__(
        self,
        code: Cell,
        data: Cell,
        query_id: int = 0,
    ) -> None:
        """Initialize JettonUpgradeBody.

        :param code: New contract code cell.
        :param data: New contract data cell.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.data = data
        self.code = code

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.JETTON_UPGRADE, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_ref(self.data)
        cell.store_ref(self.code)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> JettonUpgradeBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(
            query_id=cs.load_uint(64),
            data=cs.load_ref(),
            code=cs.load_ref(),
        )



from .ciphers import (
    AdnlChannel as AdnlChannel,
)
from .ciphers import (
    Client as Client,
)
from .ciphers import (
    Crypto as Crypto,
)
from .ciphers import (
    Server as Server,
)
from .ciphers import (
    aes_ctr_decrypt as aes_ctr_decrypt,
)
from .ciphers import (
    aes_ctr_encrypt as aes_ctr_encrypt,
)
from .ciphers import (
    create_aes_ctr_cipher as create_aes_ctr_cipher,
)
from .ciphers import (
    create_aes_ctr_sipher_from_key_n_data as create_aes_ctr_sipher_from_key_n_data,
)
from .ciphers import (
    get_key_aes_id as get_key_aes_id,
)
from .ciphers import (
    get_random as get_random,
)
from .ciphers import (
    get_shared_key as get_shared_key,
)
from .ciphers import (
    get_signature as get_signature,
)
from .crc import (
    crc16 as crc16,
)
from .crc import (
    crc32c as crc32c,
)
from .keys import (
    PBKDF_ITERATIONS as PBKDF_ITERATIONS,
)
from .keys import (
    get_secure_random_number as get_secure_random_number,
)
from .keys import (
    is_basic_seed as is_basic_seed,
)
from .keys import (
    mnemonic_is_valid as mnemonic_is_valid,
)
from .keys import (
    mnemonic_new as mnemonic_new,
)
from .keys import (
    mnemonic_to_entropy as mnemonic_to_entropy,
)
from .keys import (
    mnemonic_to_private_key as mnemonic_to_private_key,
)
from .keys import (
    mnemonic_to_seed as mnemonic_to_seed,
)
from .keys import (
    mnemonic_to_wallet_key as mnemonic_to_wallet_key,
)
from .keys import (
    private_key_to_public_key as private_key_to_public_key,
)
from .keys import (
    words as words,
)
from .signature import (
    sign_message as sign_message,
)
from .signature import (
    verify_sign as verify_sign,
)



from __future__ import annotations

from enum import Enum

__all__ = [
    "OpCode",
]


class OpCode(int, Enum):
    """Standard operation codes for TON smart contract messages."""

    TEXT_COMMENT = 0x00000000
    """Plain text comment."""
    ENCRYPTED_TEXT_COMMENT = 0x2167DA4B
    """Encrypted text comment."""
    AUTH_SIGNED_EXTERNAL = 0x7369676E
    """Signed authentication for external messages."""
    AUTH_SIGNED_INTERNAL = 0x73696E74
    """Signed authentication for internal messages."""
    OUT_ACTION_SEND_MSG = 0x0EC3C86D
    """Out action to send a message."""
    INTERNAL_TRANSFER = 0xAE42E5A4
    """Internal transfer operation."""
    TOP_UP = 0xD372158C
    """Top-up balance operation."""
    EXCESSES = 0xD53276DB
    """Return excess TON after operation."""

    NFT_TRANSFER = 0x5FCC3D14
    """Transfer ownership of an NFT."""
    NFT_OWNERSHIP_ASSIGNED = 0x05138D91
    """Notify new owner after NFT transfer."""
    NFT_GET_STATIC_DATA = 0x2FCB26A2
    """Request static data of an NFT item."""
    NFT_REPORT_STATIC_DATA = 0x8B771735
    """Response to get_static_data request."""
    NFT_EDIT_CONTENT = 0x1A0B9D51
    """Edit content of an NFT item or collection."""
    NFT_TRANSFER_EDITORSHIP = 0x1C04412A
    """Transfer editorship rights of an NFT."""
    NFT_EDITORSHIP_ASSIGNED = 0x511A4463
    """Notify new editor after editorship transfer."""
    NFT_GET_ROYALTY_PARAMS = 0x693D3950
    """Request royalty parameters of an NFT."""
    NFT_REPORT_ROYALTY_PARAMS = 0xA8CB00AD
    """Response to get_royalty_params request."""

    SBT_DESTROY = 0x1F04537A
    """Destroy a soulbound token."""
    SBT_REVOKE = 0x6F89F5E3
    """Revoke a soulbound token."""
    SBT_PROVE_OWNERSHIP = 0x04DED148
    """Prove ownership of a soulbound token."""
    SBT_REQUEST_OWNER = 0xD0C3BFEA
    """Request owner info of a soulbound token."""
    SBT_OWNERSHIP_PROOF = 0x0524C7AE
    """Response to prove_ownership request."""
    SBT_OWNER_INFO = 0x0DD607E3
    """Response to request_owner request."""

    JETTON_TRANSFER = 0xF8A7EA5
    """Transfer jettons to another address."""
    JETTON_INTERNAL_TRANSFER = 0x178D4519
    """Internal jetton transfer between wallets."""
    JETTON_TRANSFER_NOTIFICATION = 0x7362D09C
    """Notify recipient about incoming jetton transfer."""
    JETTON_BURN = 0x595F07BC
    """Burn jettons, removing them from supply."""
    JETTON_BURN_NOTIFICATION = 0x7BDD97DE
    """Notify jetton master about burn."""
    JETTON_PROVIDE_WALLET_ADDRESS = 0x2C76B973
    """Request the jetton wallet address for an owner."""
    JETTON_TAKE_WALLET_ADDRESS = 0xD1735400
    """Response to provide_wallet_address request."""
    JETTON_MINT = 0x642B7D07
    """Mint new jettons."""
    JETTON_CHANGE_ADMIN = 0x6501F354
    """Change the admin of a jetton master contract."""
    JETTON_CLAIM_ADMIN = 0xFB88E119
    """Claim admin rights for a jetton master contract."""
    JETTON_DROP_ADMIN = 0x7431F221
    """Drop admin rights from a jetton master contract."""
    JETTON_UPGRADE = 0x2508D66A
    """Upgrade the jetton contract code."""
    JETTON_CHANGE_METADATA = 0xCB862902
    """Change metadata of a jetton master contract."""

    CHANGE_DNS_RECORD = 0x4EB1F0F9
    """Change a DNS record in a TON DNS item."""
    DNS_BALANCE_RELEASE = 0x4ED14B65
    """Release balance from a TON DNS item."""

    TELEITEM_START_AUCTION = 0x487A8E81
    """Start an auction for a Telegram username NFT item."""
    TELEITEM_CANCEL_AUCTION = 0x371638AE
    """Cancel an active auction for a Telegram username NFT item."""




from .check_proof import (
    ProofError as ProofError,
)
from .check_proof import (
    calculate_node_id_short as calculate_node_id_short,
)
from .check_proof import (
    check_account_proof as check_account_proof,
)
from .check_proof import (
    check_block_header_proof as check_block_header_proof,
)
from .check_proof import (
    check_block_signatures as check_block_signatures,
)
from .check_proof import (
    check_proof as check_proof,
)
from .check_proof import (
    check_shard_proof as check_shard_proof,
)
from .check_proof import (
    compute_validator_set as compute_validator_set,
)



from pytoniq_core.boc.hashmap.utils import (
    build_edge,
    build_node,
    build_tree,
    detect_label_type,
    find_common_prefix,
    fork_map,
    is_same,
    label_long_length,
    label_same_length,
    label_short_length,
    pad,
    remove_prefix_map,
    serialize_dict,
    write_edge,
    write_label,
    write_label_long,
    write_label_same,
    write_label_short,
    write_node,
)

__all__ = [
    "build_edge",
    "build_node",
    "build_tree",
    "detect_label_type",
    "find_common_prefix",
    "fork_map",
    "is_same",
    "label_long_length",
    "label_same_length",
    "label_short_length",
    "pad",
    "remove_prefix_map",
    "serialize_dict",
    "write_edge",
    "write_label",
    "write_label_long",
    "write_label_same",
    "write_label_short",
    "write_node",
]



from pytoniq_core.tlb.vm_stack import (
    VmCellSlice,
    VmCont,
    VmControlData,
    VmError,
    VmSaveList,
    VmStack,
    VmStackList,
    VmStackValue,
    VmTuple,
    VmTupleRef,
)

__all__ = [
    "VmCellSlice",
    "VmCont",
    "VmControlData",
    "VmError",
    "VmSaveList",
    "VmStack",
    "VmStackList",
    "VmStackValue",
    "VmTuple",
    "VmTupleRef",
]



from .address import Address as Address
from .address import AddressError as AddressError
from .address import Anycast as Anycast
from .address import ExternalAddress as ExternalAddress
from .builder import Builder as Builder
from .cell import Cell as Cell
from .cell import CellError as CellError
from .exotic import CellTypes as CellTypes
from .exotic import LevelMask as LevelMask
from .hashmap import DictError as DictError
from .hashmap import HashMap as HashMap
from .hashmap import Key as Key
from .slice import Slice as Slice
from .slice import SliceError as SliceError
from .tvm_bitarray import BitarrayLike as BitarrayLike
from .tvm_bitarray import BytesLike as BytesLike
from .tvm_bitarray import TvmBitarray as TvmBitarray
from .tvm_bitarray import TvmBitarrayException as TvmBitarrayException
from .tvm_bitarray import TvmBitarrayOverflowException as TvmBitarrayOverflowException
from .tvm_bitarray import TvmBitarrayUnderflowException as TvmBitarrayUnderflowException

def begin_cell() -> Builder: ...



from collections.abc import Callable
from typing import Any

from ...boc.builder import Builder

def pad(src: str, size: int) -> str: ...
def remove_prefix_map(src: dict[str, Any], length: int) -> dict[str, Any]: ...
def find_common_prefix(src: list[str]) -> str: ...
def fork_map(src: dict[str, Any]) -> tuple[dict[str, Any], dict[str, Any]]: ...
def build_node(src: dict[str, Any]) -> dict[str, Any]: ...
def build_edge(src: dict[str, Any]) -> dict[str, Any]: ...
def build_tree(src: dict[int, Any], key_size: int) -> dict[str, Any]: ...
def write_label_short(src: str, to: Builder) -> Builder: ...
def label_short_length(src: str) -> int: ...
def write_label_long(src: str, key_length: int, to: Builder) -> Builder: ...
def label_long_length(src: str, key_length: int) -> int: ...
def write_label_same(value: bool, length: int, key_length: int, to: Builder) -> None: ...
def label_same_length(key_size: int) -> int: ...
def is_same(src: str) -> bool: ...
def detect_label_type(src: str, key_size: int) -> str: ...
def write_label(src: str, key_size: int, to: Builder) -> None: ...
def write_node(src: dict[str, Any], key_size: int, serializer: Callable[..., Any], to: Builder) -> None: ...
def write_edge(src: dict[str, Any], key_size: int, serializer: Callable[..., Any], to: Builder) -> None: ...
def serialize_dict(src: dict[int, Any], key_size: int, serializer: Callable[..., Any]) -> Builder: ...



from pytoniq_core.boc.deserialize import (
    SERIALIZED_BOC_IDX_CRC32C,
    SERIALIZED_BOC_IDX_PREFIX,
    SERIALIZED_BOC_PREFIX,
    Boc,
    BocError,
    NullCell,
)

__all__ = [
    "SERIALIZED_BOC_IDX_CRC32C",
    "SERIALIZED_BOC_IDX_PREFIX",
    "SERIALIZED_BOC_PREFIX",
    "Boc",
    "BocError",
    "NullCell",
]



from collections.abc import Callable
from typing import Any

from ton_core.boc.cell import Cell
from ton_core.boc.slice import Slice
from ton_core.tlb.tlb import TlbScheme

class HashUpdate(TlbScheme):
    old_hash: bytes
    new_hash: bytes
    def __init__(self, old_hash: bytes, new_hash: bytes) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> HashUpdate: ...

class MerkleUpdate(TlbScheme):
    cell: Cell
    old_hash: bytes
    new_hash: bytes
    old: object
    new: object
    def __init__(self, cell: Cell, old_hash: bytes, new_hash: bytes, old: object, new: object) -> None: ...
    @classmethod
    def serialize(cls, *args: Any) -> None: ...
    @classmethod
    def deserialize(cls, cell: Cell, deserializer: Callable[..., Any]) -> MerkleUpdate | None: ...

def deserialize_shard_hashes(cell_slice: Slice) -> dict[Any, Any] | None: ...

def uint64_to_int64(num: int) -> int: ...



from .configs import (
    BaseContractConfig,
    BaseWalletConfig,
    WalletHighloadV2Config,
    WalletHighloadV3Config,
    WalletPreprocessedV2Config,
    WalletV1Config,
    WalletV2Config,
    WalletV3Config,
    WalletV4Config,
    WalletV5BetaConfig,
    WalletV5Config,
)
from .params import (
    BaseWalletParams,
    WalletHighloadV2Params,
    WalletHighloadV3Params,
    WalletPreprocessedV2Params,
    WalletV1Params,
    WalletV2Params,
    WalletV3Params,
    WalletV4Params,
    WalletV5BetaParams,
    WalletV5Params,
)
from .tlb import (
    BaseWalletData,
    EncryptedTextCommentBody,
    OutActionSendMsg,
    TextCommentBody,
    WalletHighloadV2Data,
    WalletHighloadV3Data,
    WalletPreprocessedV2Data,
    WalletV1Data,
    WalletV2Data,
    WalletV3Data,
    WalletV4Data,
    WalletV5BetaData,
    WalletV5Data,
    WalletV5SubwalletID,
)

__all__ = [
    "BaseContractConfig",
    "BaseWalletConfig",
    "BaseWalletData",
    "BaseWalletParams",
    "EncryptedTextCommentBody",
    "OutActionSendMsg",
    "TextCommentBody",
    "WalletHighloadV2Config",
    "WalletHighloadV2Data",
    "WalletHighloadV2Params",
    "WalletHighloadV3Config",
    "WalletHighloadV3Data",
    "WalletHighloadV3Params",
    "WalletPreprocessedV2Config",
    "WalletPreprocessedV2Data",
    "WalletPreprocessedV2Params",
    "WalletV1Config",
    "WalletV1Data",
    "WalletV1Params",
    "WalletV2Config",
    "WalletV2Data",
    "WalletV2Params",
    "WalletV3Config",
    "WalletV3Data",
    "WalletV3Params",
    "WalletV4Config",
    "WalletV4Data",
    "WalletV4Params",
    "WalletV5BetaConfig",
    "WalletV5BetaData",
    "WalletV5BetaParams",
    "WalletV5Config",
    "WalletV5Data",
    "WalletV5Params",
    "WalletV5SubwalletID",
]



from typing import Any, Literal

from ton_core.boc.address import Address
from ton_core.boc.cell import Cell
from ton_core.boc.slice import Slice
from ton_core.tlb.tlb import TlbError, TlbScheme

class Account(TlbScheme):
    addr: Address
    storage_stat: StorageInfo
    storage: AccountStorage
    def __init__(self, addr: Address, storage_stat: StorageInfo, storage: AccountStorage) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> Account | None: ...

class AccountBlock(TlbScheme):
    account_addr: str
    transactions: tuple[dict[Any, Any], list[Any]]
    state_update: object
    def __init__(self, account_addr: str, transactions: tuple[dict[Any, Any], list[Any]], state_update: object) -> None: ...
    @classmethod
    def serialize(cls, *args: object) -> None: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> AccountBlock: ...

class AccountError(TlbError): ...

class AccountState(TlbScheme):
    type_: str
    state_init: StateInit
    state_hash: bytes
    def __init__(self, type_: str, **kwargs: object) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> AccountState: ...

class AccountStatus(TlbScheme):
    type_: Literal["uninitialized", "frozen", "active", "nonexist"]
    def __init__(self, type_: Literal["uninitialized", "frozen", "active", "nonexist"]) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> AccountStatus: ...

class AccountStorage(TlbScheme):
    last_trans_lt: int
    balance: object
    state: AccountState
    def __init__(self, last_trans_lt: int, balance: object, state: AccountState) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> AccountStorage: ...

class ShardAccount(TlbScheme):
    account: Account | None
    last_trans_hash: bytes
    last_trans_lt: int
    cell: Cell | None
    def __init__(self, account: Account | None, last_trans_hash: bytes, last_trans_lt: int, cell: Cell | None = ...) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> ShardAccount: ...

class SimpleAccount(TlbScheme):
    address: Address
    balance: int
    state: SimpleAccountState
    def __init__(self, address: Address, balance: int, state: SimpleAccountState) -> None: ...
    @classmethod
    def serialize(cls, *args: object) -> None: ...
    @classmethod
    def deserialize(cls, *args: object) -> None: ...
    @classmethod
    def from_raw(cls, account: Account | None, address: Address | None = ...) -> SimpleAccount: ...
    def is_uninitialized(self) -> bool: ...
    def is_frozen(self) -> bool: ...
    def is_active(self) -> bool: ...

class SimpleAccountState(TlbScheme):
    type_: Literal["uninitialized", "frozen", "active"]
    state_hash: bytes | None
    state_init: StateInit | None
    def __init__(self, type_: Literal["uninitialized", "frozen", "active"], state_hash: bytes | None = ..., state_init: StateInit | None = ...) -> None: ...
    @classmethod
    def serialize(cls, *args: object) -> None: ...
    @classmethod
    def deserialize(cls, *args: object) -> None: ...
    @classmethod
    def from_raw(cls, account_state: AccountState | None) -> SimpleAccountState: ...

class StateInit(TlbScheme):
    split_depth: int | None
    special: TickTock | None
    code: Cell | None
    data: Cell | None
    library: Cell | None
    def __init__(self, split_depth: int | None = ..., special: TickTock | None = ..., code: Cell | None = ..., data: Cell | None = ..., library: Cell | None = ...) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> StateInit: ...

class StorageExtraInfo(TlbScheme):
    type_: Literal["storage_extra_none", "storage_extra_info"]
    dict_hash: bytes | None
    def __init__(self, type_: Literal["storage_extra_none", "storage_extra_info"], dict_hash: bytes | None = ...) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> StorageExtraInfo: ...

class StorageInfo(TlbScheme):
    used: StorageUsed
    last_paid: int
    storage_extra: StorageExtraInfo
    due_payment: int | None
    def __init__(self, used: StorageUsed, storage_extra: StorageExtraInfo, last_paid: int, due_payment: int | None) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> StorageInfo: ...

class StorageUsed(TlbScheme):
    cells: int
    bits: int
    public_cells: int | None
    def __init__(self, cells: int, bits: int, *args: object, **kwargs: object) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> StorageUsed: ...

class StorageUsedShort(TlbScheme):
    cells: int
    bits: int
    def __init__(self, cells: int, bits: int) -> None: ...
    @classmethod
    def serialize(cls, *args: object) -> None: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> StorageUsedShort: ...

class TickTock(TlbScheme):
    tick: bool
    tock: bool
    def __init__(self, tick: bool, tock: bool) -> None: ...
    def serialize(self) -> Cell: ...
    @classmethod
    def deserialize(cls, cell_slice: Slice) -> TickTock: ...



from typing import Any

from .builder import Builder
from .cell import Cell
from .slice import Slice
from .tvm_bitarray import TvmBitarray

SERIALIZED_BOC_IDX_CRC32C: bytes
SERIALIZED_BOC_IDX_PREFIX: bytes
SERIALIZED_BOC_PREFIX: bytes

class BocError(Exception): ...

class NullCell:
    bits: TvmBitarray
    refs: list[Any]
    type_: int
    def __init__(self, bits: TvmBitarray, refs: list[Any], type_: int) -> None: ...
    def to_cell(self) -> Cell: ...
    def to_slice(self) -> Slice: ...
    def to_builder(self) -> Builder: ...
    def copy(self) -> NullCell: ...
    def __repr__(self) -> str: ...
    def __str__(self, t: int = ..., comma: bool = ...) -> str: ...

class Boc:
    data: bytes
    data_len: int
    def __init__(self, data: bytes | str) -> None: ...
    @classmethod
    def from_base64(cls, data: str) -> Boc: ...
    @classmethod
    def from_hex(cls, data: str) -> Boc: ...
    def parse(self) -> None: ...
    @staticmethod
    def deserialize_boc_header(data: bytes) -> dict[str, Any]: ...
    @staticmethod
    def deserialize_cell(data: bytes, ref_index_size: int) -> tuple[dict[str, Any], int]: ...
    def deserialize(self, cls: type | None = ...) -> list[Any]: ...



from __future__ import annotations

import abc
from typing import Any, ClassVar, cast

from ton_core.boc import Address, Cell, HashMap, Slice, begin_cell
from ton_core.contrib.contracts._utils import _load_std_address
from ton_core.contrib.contracts.nft.tlb import OffchainContent, OnchainContent
from ton_core.contrib.contracts.opcodes import OpCode
from ton_core.contrib.types import ADNL, AddressLike, BagID, Binary, BinaryLike, DNSCategory, DNSPrefix
from ton_core.contrib.utils import string_hash
from ton_core.tlb import TlbScheme

__all__ = [
    "BaseDNSRecord",
    "BaseDNSRecordAddress",
    "BaseDNSRecordBinary",
    "ChangeDNSRecordBody",
    "DNSBalanceReleaseBody",
    "DNSRecordDNSNextResolver",
    "DNSRecordSite",
    "DNSRecordStorage",
    "DNSRecordText",
    "DNSRecordWallet",
    "DNSRecords",
    "RenewDNSBody",
    "TONDNSAuction",
    "TONDNSCollectionData",
    "TONDNSItemData",
]


class BaseDNSRecord(TlbScheme, abc.ABC):
    """Abstract base for DNS record types."""

    PREFIX: ClassVar[DNSPrefix]

    def __init__(self, value: Any) -> None:
        """Initialize BaseDNSRecord.

        :param value: Record value (type depends on subclass).
        """
        self.value = value

    @abc.abstractmethod
    def _build_cell(self) -> Cell:
        """Build Cell from record value."""

    @classmethod
    @abc.abstractmethod
    def _parse_cell(cls, cs: Slice) -> Any:
        """Parse record value from Slice."""

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        return self._build_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> Any:
        """Deserialize from Slice."""
        return cls._parse_cell(cs)


class BaseDNSRecordAddress(BaseDNSRecord):
    """Base for DNS records containing TON addresses."""

    def __init__(self, value: Address | str) -> None:
        """Initialize BaseDNSRecordAddress.

        :param value: TON address.
        """
        if not isinstance(value, Address):
            value = Address(value)
        super().__init__(value)

    def _build_cell(self) -> Cell:
        cell = begin_cell()
        cell.store_uint(self.PREFIX, 16)
        cell.store_address(self.value)
        cell.store_uint(0, 8)
        return cell.end_cell()

    @classmethod
    def _parse_cell(cls, cs: Slice) -> BaseDNSRecordAddress:
        cs.skip_bits(16)
        addr = cs.load_address()
        if not isinstance(addr, Address):
            raise ValueError(f"Expected Address, got {type(addr).__name__}")
        return cls(addr)


class BaseDNSRecordBinary(BaseDNSRecord):
    """Base for DNS records containing binary data."""

    BINARY_CLS: ClassVar[type[Binary]]

    def __init__(self, value: Binary | BinaryLike) -> None:
        """Initialize BaseDNSRecordBinary.

        :param value: Binary data.
        """
        if not isinstance(value, Binary):
            value = self.__class__.BINARY_CLS(value)
        super().__init__(value)

    def _build_cell(self) -> Cell:
        cell = begin_cell()
        cell.store_uint(self.PREFIX, 16)
        cell.store_bytes(self.value.as_bytes)
        cell.store_uint(0, 8)
        return cell.end_cell()

    @classmethod
    def _parse_cell(cls, cs: Slice) -> BaseDNSRecordBinary:
        cs.skip_bits(16)
        return cls(cs.load_bytes(32))


class DNSRecordDNSNextResolver(BaseDNSRecordAddress):
    """DNS record pointing to next resolver contract."""

    PREFIX: ClassVar[DNSPrefix] = DNSPrefix.DNS_NEXT_RESOLVER

    def _build_cell(self) -> Cell:
        cell = begin_cell()
        cell.store_uint(self.PREFIX, 16)
        cell.store_address(self.value)
        return cell.end_cell()

    @classmethod
    def _parse_cell(cls, cs: Slice) -> DNSRecordDNSNextResolver:
        cs.skip_bits(16)
        addr = cs.load_address()
        if not isinstance(addr, Address):
            raise ValueError(f"Expected Address, got {type(addr).__name__}")
        return cls(addr)

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSRecordDNSNextResolver:
        """Deserialize from Slice."""
        return cast("DNSRecordDNSNextResolver", super().deserialize(cs))


class DNSRecordWallet(BaseDNSRecordAddress):
    """DNS record pointing to wallet address."""

    PREFIX: ClassVar[DNSPrefix] = DNSPrefix.WALLET

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSRecordWallet:
        """Deserialize from Slice."""
        return cast("DNSRecordWallet", super().deserialize(cs))


class DNSRecordStorage(BaseDNSRecordBinary):
    """DNS record pointing to TON Storage bag."""

    PREFIX: ClassVar[DNSPrefix] = DNSPrefix.STORAGE
    BINARY_CLS: ClassVar[type[BagID]] = BagID

    def _build_cell(self) -> Cell:
        cell = begin_cell()
        cell.store_uint(self.PREFIX, 16)
        cell.store_bytes(self.value.as_bytes)
        return cell.end_cell()

    @classmethod
    def _parse_cell(cls, cs: Slice) -> DNSRecordStorage:
        cs.skip_bits(16)
        return cls(cs.load_bytes(32))

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSRecordStorage:
        """Deserialize from Slice."""
        return cast("DNSRecordStorage", super().deserialize(cs))


class DNSRecordSite(BaseDNSRecordBinary):
    """DNS record pointing to TON Site (ADNL address)."""

    PREFIX: ClassVar[DNSPrefix] = DNSPrefix.SITE
    BINARY_CLS: ClassVar[type[ADNL]] = ADNL

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSRecordSite:
        """Deserialize from Slice."""
        return cast("DNSRecordSite", super().deserialize(cs))


class DNSRecordText(BaseDNSRecord):
    """DNS text record."""

    PREFIX: ClassVar[DNSPrefix] = DNSPrefix.TEXT

    def __init__(self, value: str) -> None:
        """Initialize DNSRecordText.

        :param value: Text content.
        """
        super().__init__(value)

    def _build_cell(self) -> Cell:
        cell = begin_cell()
        cell.store_uint(self.PREFIX, 16)
        cell.store_snake_string(self.value)
        return cell.end_cell()

    @classmethod
    def _parse_cell(cls, cs: Slice) -> DNSRecordText:
        cs.skip_bits(16)
        return cls(cs.load_snake_string())

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSRecordText:
        """Deserialize from Slice."""
        return cast("DNSRecordText", super().deserialize(cs))


class DNSRecords(OnchainContent):
    """Collection of DNS records for a domain."""

    _DNS_RECORDS_CLASSES: ClassVar[dict[str, type[BaseDNSRecord]]] = {
        "dns_next_resolver": DNSRecordDNSNextResolver,
        "storage": DNSRecordStorage,
        "wallet": DNSRecordWallet,
        "site": DNSRecordSite,
        "text": DNSRecordText,
    }

    _DNS_CATEGORIES: ClassVar[dict[int, str]] = {
        DNSCategory.DNS_NEXT_RESOLVER: "dns_next_resolver",
        DNSCategory.STORAGE: "storage",
        DNSCategory.WALLET: "wallet",
        DNSCategory.SITE: "site",
        DNSCategory.TEXT: "text",
    }

    _DNS_KEYS: ClassVar[set[str]] = set(_DNS_RECORDS_CLASSES.keys())

    def __init__(self, data: dict[str | int, Any]) -> None:
        """Initialize DNSRecords.

        :param data: Record keys to values.
        """
        self.records: dict[str | int, BaseDNSRecord] = {}

        other: dict[str | int, Any] = {}

        for raw_key, val in data.items():
            if isinstance(raw_key, int) and raw_key in self._DNS_CATEGORIES:
                key: str = self._DNS_CATEGORIES[raw_key]
            elif isinstance(raw_key, str):
                key = raw_key
            else:
                other[raw_key] = val
                continue
            if key not in self._DNS_KEYS:
                other[raw_key] = val
                continue
            if not isinstance(val, BaseDNSRecord):
                record_cls = self._DNS_RECORDS_CLASSES[key]
                val = self._to_record(record_cls, val)
            self.records[key] = val

        super().__init__(other)

    @classmethod
    def _to_record(
        cls, record_cls: type[BaseDNSRecord], val: Any
    ) -> BaseDNSRecord:
        if isinstance(val, Cell):
            cs = val.begin_parse().load_ref().begin_parse()
            return cast("BaseDNSRecord", record_cls.deserialize(cs))
        return record_cls(val)

    def _build_hashmap(self) -> HashMap:
        hashmap = super()._build_hashmap()
        for key, val in self.records.items():
            if isinstance(key, str):
                key = string_hash(key)
            hashmap.set_int_key(key, val.serialize())
        return hashmap

    @classmethod
    def _parse_hashmap(
        cls,
        hashmap: dict[str | int, Cell],
    ) -> dict[str | int, Any]:
        hashmap = super()._parse_hashmap(hashmap)
        for key in cls._DNS_KEYS:
            int_key = string_hash(key)
            if int_key in hashmap:
                val = hashmap.pop(int_key)
                cs = val.begin_parse().load_ref().begin_parse()
                hashmap[key] = cls._DNS_RECORDS_CLASSES[key].deserialize(cs)
        return hashmap


class TONDNSAuction(TlbScheme):
    """Auction state for a TON DNS domain."""

    def __init__(
        self,
        max_bid_address: AddressLike | None,
        max_bid_amount: int,
        auction_end_time: int,
    ) -> None:
        """Initialize TONDNSAuction.

        :param max_bid_address: Highest bidder address.
        :param max_bid_amount: Highest bid in nanotons.
        :param auction_end_time: Auction end unix timestamp.
        """
        self.max_bid_address = max_bid_address
        self.max_bid_amount = max_bid_amount
        self.auction_end_time = auction_end_time

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_address(self.max_bid_address)
        cell.store_coins(self.max_bid_amount)
        cell.store_uint(self.auction_end_time, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> TONDNSAuction:
        """Deserialize from Slice."""
        return cls(
            max_bid_address=_load_std_address(cs),
            max_bid_amount=cs.load_coins() or 0,
            auction_end_time=cs.load_uint(64),
        )


class TONDNSCollectionData(TlbScheme):
    """On-chain data for TON DNS collection contract."""

    def __init__(
        self,
        content: OffchainContent,
        nft_item_code: Cell,
    ) -> None:
        """Initialize TONDNSCollectionData.

        :param content: Off-chain collection metadata.
        :param nft_item_code: Code cell for DNS item contracts.
        """
        self.content = content
        self.nft_item_code = nft_item_code

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_ref(self.content.serialize(True))
        cell.store_ref(self.nft_item_code)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> TONDNSCollectionData:
        """Deserialize from Slice."""
        return cls(
            content=OffchainContent.deserialize(cs.load_ref().begin_parse(), True),
            nft_item_code=cs.load_ref(),
        )


class TONDNSItemData(TlbScheme):
    """On-chain data for TON DNS item (domain) contract."""

    def __init__(
        self,
        index: int,
        collection_address: AddressLike | None,
        owner_address: AddressLike | None,
        content: OnchainContent,
        domain: str,
        last_fill_up_time: int,
        auction: TONDNSAuction | None = None,
    ) -> None:
        """Initialize TONDNSItemData.

        :param index: Item index within collection.
        :param collection_address: Parent collection address.
        :param owner_address: Current domain owner address.
        :param content: On-chain DNS records and metadata.
        :param domain: Domain name string.
        :param last_fill_up_time: Last renewal unix timestamp.
        :param auction: Active auction state, or None.
        """
        self.index = index
        self.collection_address = collection_address
        self.owner_address = owner_address
        self.content = content
        self.domain = domain
        self.auction = auction
        self.last_fill_up_time = last_fill_up_time

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(self.index, 256)
        cell.store_address(self.collection_address)
        cell.store_address(self.owner_address)
        cell.store_ref(self.content.serialize(True))
        cell.store_ref(begin_cell().store_snake_string(self.domain).end_cell())
        cell.store_dict(self.auction.serialize() if self.auction else None)
        cell.store_uint(self.last_fill_up_time, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> TONDNSItemData:
        """Deserialize from Slice."""
        return cls(
            index=cs.load_uint(256),
            collection_address=_load_std_address(cs),
            owner_address=_load_std_address(cs),
            content=OnchainContent.deserialize(cs.load_ref().begin_parse(), True),
            domain=cs.load_ref().begin_parse().load_snake_string(),
            auction=(
                TONDNSAuction.deserialize(auction_cell.begin_parse())
                if (auction_cell := cs.load_maybe_ref()) is not None
                else None
            ),
            last_fill_up_time=cs.load_uint(64),
        )


class ChangeDNSRecordBody(TlbScheme):
    """Message body for changing a DNS record."""

    def __init__(
        self,
        category: DNSCategory,
        record: DNSRecordDNSNextResolver | DNSRecordSite | DNSRecordStorage | DNSRecordWallet | None = None,
        query_id: int = 0,
    ) -> None:
        """Initialize ChangeDNSRecordBody.

        :param category: DNS record category to change.
        :param record: New record value, or None to delete.
        :param query_id: Query identifier.
        """
        self.query_id = query_id
        self.category = category
        self.record = record

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.CHANGE_DNS_RECORD, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_uint(self.category.value, 256)
        if self.record is not None:
            cell.store_ref(self.record.serialize())
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> ChangeDNSRecordBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        query_id = cs.load_uint(64)
        category_value = cs.load_uint(256)
        record = None
        if cs.remaining_refs > 0:
            record_cs = cs.load_ref().begin_parse()
            prefix = record_cs.preload_uint(16)
            record_map: dict[int, type[BaseDNSRecord]] = {
                DNSPrefix.DNS_NEXT_RESOLVER: DNSRecordDNSNextResolver,
                DNSPrefix.WALLET: DNSRecordWallet,
                DNSPrefix.STORAGE: DNSRecordStorage,
                DNSPrefix.SITE: DNSRecordSite,
                DNSPrefix.TEXT: DNSRecordText,
            }
            record_cls = record_map.get(prefix)
            if record_cls is not None:
                record = record_cls.deserialize(record_cs)
        return cls(
            query_id=query_id,
            category=DNSCategory(category_value),
            record=record,
        )


class RenewDNSBody(TlbScheme):
    """Message body for renewing a DNS domain (change_dns_record with category=0 and no value)."""

    def __init__(self, query_id: int = 0) -> None:
        """Initialize RenewDNSBody.

        :param query_id: Query identifier.
        """
        self.query_id = query_id

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.CHANGE_DNS_RECORD, 32)
        cell.store_uint(self.query_id, 64)
        cell.store_uint(0, 256)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> RenewDNSBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        query_id = cs.load_uint(64)
        cs.skip_bits(256)
        return cls(query_id=query_id)


class DNSBalanceReleaseBody(TlbScheme):
    """Message body for releasing DNS domain balance."""

    def __init__(self, query_id: int = 0) -> None:
        """Initialize DNSBalanceReleaseBody.

        :param query_id: Query identifier.
        """
        self.query_id = query_id

    def serialize(self) -> Cell:
        """Serialize to Cell."""
        cell = begin_cell()
        cell.store_uint(OpCode.DNS_BALANCE_RELEASE, 32)
        cell.store_uint(self.query_id, 64)
        return cell.end_cell()

    @classmethod
    def deserialize(cls, cs: Slice) -> DNSBalanceReleaseBody:
        """Deserialize from Slice."""
        cs.skip_bits(32)
        return cls(query_id=cs.load_uint(64))

